"""
Python module generated from Java source file java.lang.invoke.MethodType

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.lang.constant import ClassDesc
from java.lang.constant import Constable
from java.lang.constant import MethodTypeDesc
from java.lang.invoke import *
from java.lang.ref import Reference
from java.lang.ref import ReferenceQueue
from java.lang.ref import WeakReference
from java.util import Arrays
from java.util import Collections
from java.util import NoSuchElementException
from java.util import Objects
from java.util import Optional
from java.util import StringJoiner
from java.util.concurrent import ConcurrentHashMap
from java.util.concurrent import ConcurrentMap
from java.util.stream import Stream
from jdk.internal.vm.annotation import Stable
from sun.invoke.util import BytecodeDescriptor
from sun.invoke.util import VerifyType
from sun.invoke.util import Wrapper
from sun.security.util import SecurityConstants
from typing import Any, Callable, Iterable, Tuple


class MethodType(Constable, OfMethod, Serializable):
    """
    A method type represents the arguments and return type accepted and
    returned by a method handle, or the arguments and return type passed
    and expected  by a method handle caller.  Method types must be properly
    matched between a method handle and all its callers,
    and the JVM's operations enforce this matching at, specifically
    during calls to MethodHandle.invokeExact MethodHandle.invokeExact
    and MethodHandle.invoke MethodHandle.invoke, and during execution
    of `invokedynamic` instructions.
    
    The structure is a return type accompanied by any number of parameter types.
    The types (primitive, `void`, and reference) are represented by Class objects.
    (For ease of exposition, we treat `void` as if it were a type.
    In fact, it denotes the absence of a return type.)
    
    All instances of `MethodType` are immutable.
    Two instances are completely interchangeable if they compare equal.
    Equality depends on pairwise correspondence of the return and parameter types and on nothing else.
    
    This type can be created only by factory methods.
    All factory methods may cache values, though caching is not guaranteed.
    Some factory methods are static, while others are virtual methods which
    modify precursor method types, e.g., by changing a selected parameter.
    
    Factory methods which operate on groups of parameter types
    are systematically presented in two versions, so that both Java arrays and
    Java lists can be used to work with groups of parameter types.
    The query methods `parameterArray` and `parameterList`
    also provide a choice between arrays and lists.
    
    `MethodType` objects are sometimes derived from bytecode instructions
    such as `invokedynamic`, specifically from the type descriptor strings associated
    with the instructions in a class file's constant pool.
    
    Like classes and strings, method types can also be represented directly
    in a class file's constant pool as constants.
    A method type may be loaded by an `ldc` instruction which refers
    to a suitable `CONSTANT_MethodType` constant pool entry.
    The entry refers to a `CONSTANT_Utf8` spelling for the descriptor string.
    (For full details on method type constants, see sections 4.4.8 and 5.4.3.5 of the Java Virtual Machine
    Specification.)
    
    When the JVM materializes a `MethodType` from a descriptor string,
    all classes named in the descriptor must be accessible, and will be loaded.
    (But the classes need not be initialized, as is the case with a `CONSTANT_Class`.)
    This loading may occur at any time before the `MethodType` object is first derived.
    
    **<a id="descriptor">Nominal Descriptors</a>**
    
    A `MethodType` can be described in MethodTypeDesc nominal form
    if and only if all of the parameter types and return type can be described
    with a Class.describeConstable() nominal descriptor represented by
    ClassDesc.  If a method type can be described nominally, then:
    
    - The method type has a MethodTypeDesc nominal descriptor
        returned by .describeConstable() MethodType::describeConstable.
    - The descriptor string returned by
        .descriptorString() MethodType::descriptorString or
        .toMethodDescriptorString() MethodType::toMethodDescriptorString
        for the method type is a method descriptor (JVMS 4.3.3).
    
    
    If any of the parameter types or return type cannot be described
    nominally, i.e. Class.describeConstable() Class::describeConstable
    returns an empty optional for that type,
    then the method type cannot be described nominally:
    
    - The method type has no MethodTypeDesc nominal descriptor and
        .describeConstable() MethodType::describeConstable returns
        an empty optional.
    - The descriptor string returned by
        .descriptorString() MethodType::descriptorString or
        .toMethodDescriptorString() MethodType::toMethodDescriptorString
        for the method type is not a type descriptor.

    Author(s)
    - John Rose, JSR 292 EG

    Since
    - 1.7
    """

    @staticmethod
    def methodType(rtype: type[Any], ptypes: list[type[Any]]) -> "MethodType":
        """
        Finds or creates an instance of the given method type.

        Arguments
        - rtype: the return type
        - ptypes: the parameter types

        Returns
        - a method type with the given components

        Raises
        - NullPointerException: if `rtype` or `ptypes` or any element of `ptypes` is null
        - IllegalArgumentException: if any element of `ptypes` is `void.class`
        """
        ...


    @staticmethod
    def methodType(rtype: type[Any], ptypes: list[type[Any]]) -> "MethodType":
        """
        Finds or creates a method type with the given components.
        Convenience method for .methodType(java.lang.Class, java.lang.Class[]) methodType.

        Arguments
        - rtype: the return type
        - ptypes: the parameter types

        Returns
        - a method type with the given components

        Raises
        - NullPointerException: if `rtype` or `ptypes` or any element of `ptypes` is null
        - IllegalArgumentException: if any element of `ptypes` is `void.class`
        """
        ...


    @staticmethod
    def methodType(rtype: type[Any], ptype0: type[Any], *ptypes: Tuple[type[Any], ...]) -> "MethodType":
        """
        Finds or creates a method type with the given components.
        Convenience method for .methodType(java.lang.Class, java.lang.Class[]) methodType.
        The leading parameter type is prepended to the remaining array.

        Arguments
        - rtype: the return type
        - ptype0: the first parameter type
        - ptypes: the remaining parameter types

        Returns
        - a method type with the given components

        Raises
        - NullPointerException: if `rtype` or `ptype0` or `ptypes` or any element of `ptypes` is null
        - IllegalArgumentException: if `ptype0` or `ptypes` or any element of `ptypes` is `void.class`
        """
        ...


    @staticmethod
    def methodType(rtype: type[Any]) -> "MethodType":
        """
        Finds or creates a method type with the given components.
        Convenience method for .methodType(java.lang.Class, java.lang.Class[]) methodType.
        The resulting method has no parameter types.

        Arguments
        - rtype: the return type

        Returns
        - a method type with the given return value

        Raises
        - NullPointerException: if `rtype` is null
        """
        ...


    @staticmethod
    def methodType(rtype: type[Any], ptype0: type[Any]) -> "MethodType":
        """
        Finds or creates a method type with the given components.
        Convenience method for .methodType(java.lang.Class, java.lang.Class[]) methodType.
        The resulting method has the single given parameter type.

        Arguments
        - rtype: the return type
        - ptype0: the parameter type

        Returns
        - a method type with the given return value and parameter type

        Raises
        - NullPointerException: if `rtype` or `ptype0` is null
        - IllegalArgumentException: if `ptype0` is `void.class`
        """
        ...


    @staticmethod
    def methodType(rtype: type[Any], ptypes: "MethodType") -> "MethodType":
        """
        Finds or creates a method type with the given components.
        Convenience method for .methodType(java.lang.Class, java.lang.Class[]) methodType.
        The resulting method has the same parameter types as `ptypes`,
        and the specified return type.

        Arguments
        - rtype: the return type
        - ptypes: the method type which supplies the parameter types

        Returns
        - a method type with the given components

        Raises
        - NullPointerException: if `rtype` or `ptypes` is null
        """
        ...


    @staticmethod
    def genericMethodType(objectArgCount: int, finalArray: bool) -> "MethodType":
        """
        Finds or creates a method type whose components are `Object` with an optional trailing `Object[]` array.
        Convenience method for .methodType(java.lang.Class, java.lang.Class[]) methodType.
        All parameters and the return type will be `Object`,
        except the final array parameter if any, which will be `Object[]`.

        Arguments
        - objectArgCount: number of parameters (excluding the final array parameter if any)
        - finalArray: whether there will be a trailing array parameter, of type `Object[]`

        Returns
        - a generally applicable method type, for all calls of the given fixed argument count and a collected array of further arguments

        Raises
        - IllegalArgumentException: if `objectArgCount` is negative or greater than 255 (or 254, if `finalArray` is True)

        See
        - .genericMethodType(int)
        """
        ...


    @staticmethod
    def genericMethodType(objectArgCount: int) -> "MethodType":
        """
        Finds or creates a method type whose components are all `Object`.
        Convenience method for .methodType(java.lang.Class, java.lang.Class[]) methodType.
        All parameters and the return type will be Object.

        Arguments
        - objectArgCount: number of parameters

        Returns
        - a generally applicable method type, for all calls of the given argument count

        Raises
        - IllegalArgumentException: if `objectArgCount` is negative or greater than 255

        See
        - .genericMethodType(int, boolean)
        """
        ...


    def changeParameterType(self, num: int, nptype: type[Any]) -> "MethodType":
        """
        Finds or creates a method type with a single different parameter type.
        Convenience method for .methodType(java.lang.Class, java.lang.Class[]) methodType.

        Arguments
        - num: the index (zero-based) of the parameter type to change
        - nptype: a new parameter type to replace the old one with

        Returns
        - the same type, except with the selected parameter changed

        Raises
        - IndexOutOfBoundsException: if `num` is not a valid index into `parameterArray()`
        - IllegalArgumentException: if `nptype` is `void.class`
        - NullPointerException: if `nptype` is null
        """
        ...


    def insertParameterTypes(self, num: int, *ptypesToInsert: Tuple[type[Any], ...]) -> "MethodType":
        """
        Finds or creates a method type with additional parameter types.
        Convenience method for .methodType(java.lang.Class, java.lang.Class[]) methodType.

        Arguments
        - num: the position (zero-based) of the inserted parameter type(s)
        - ptypesToInsert: zero or more new parameter types to insert into the parameter list

        Returns
        - the same type, except with the selected parameter(s) inserted

        Raises
        - IndexOutOfBoundsException: if `num` is negative or greater than `parameterCount()`
        - IllegalArgumentException: if any element of `ptypesToInsert` is `void.class`
                                         or if the resulting method type would have more than 255 parameter slots
        - NullPointerException: if `ptypesToInsert` or any of its elements is null
        """
        ...


    def appendParameterTypes(self, *ptypesToInsert: Tuple[type[Any], ...]) -> "MethodType":
        """
        Finds or creates a method type with additional parameter types.
        Convenience method for .methodType(java.lang.Class, java.lang.Class[]) methodType.

        Arguments
        - ptypesToInsert: zero or more new parameter types to insert after the end of the parameter list

        Returns
        - the same type, except with the selected parameter(s) appended

        Raises
        - IllegalArgumentException: if any element of `ptypesToInsert` is `void.class`
                                         or if the resulting method type would have more than 255 parameter slots
        - NullPointerException: if `ptypesToInsert` or any of its elements is null
        """
        ...


    def insertParameterTypes(self, num: int, ptypesToInsert: list[type[Any]]) -> "MethodType":
        """
        Finds or creates a method type with additional parameter types.
        Convenience method for .methodType(java.lang.Class, java.lang.Class[]) methodType.

        Arguments
        - num: the position (zero-based) of the inserted parameter type(s)
        - ptypesToInsert: zero or more new parameter types to insert into the parameter list

        Returns
        - the same type, except with the selected parameter(s) inserted

        Raises
        - IndexOutOfBoundsException: if `num` is negative or greater than `parameterCount()`
        - IllegalArgumentException: if any element of `ptypesToInsert` is `void.class`
                                         or if the resulting method type would have more than 255 parameter slots
        - NullPointerException: if `ptypesToInsert` or any of its elements is null
        """
        ...


    def appendParameterTypes(self, ptypesToInsert: list[type[Any]]) -> "MethodType":
        """
        Finds or creates a method type with additional parameter types.
        Convenience method for .methodType(java.lang.Class, java.lang.Class[]) methodType.

        Arguments
        - ptypesToInsert: zero or more new parameter types to insert after the end of the parameter list

        Returns
        - the same type, except with the selected parameter(s) appended

        Raises
        - IllegalArgumentException: if any element of `ptypesToInsert` is `void.class`
                                         or if the resulting method type would have more than 255 parameter slots
        - NullPointerException: if `ptypesToInsert` or any of its elements is null
        """
        ...


    def dropParameterTypes(self, start: int, end: int) -> "MethodType":
        """
        Finds or creates a method type with some parameter types omitted.
        Convenience method for .methodType(java.lang.Class, java.lang.Class[]) methodType.

        Arguments
        - start: the index (zero-based) of the first parameter type to remove
        - end: the index (greater than `start`) of the first parameter type after not to remove

        Returns
        - the same type, except with the selected parameter(s) removed

        Raises
        - IndexOutOfBoundsException: if `start` is negative or greater than `parameterCount()`
                                         or if `end` is negative or greater than `parameterCount()`
                                         or if `start` is greater than `end`
        """
        ...


    def changeReturnType(self, nrtype: type[Any]) -> "MethodType":
        """
        Finds or creates a method type with a different return type.
        Convenience method for .methodType(java.lang.Class, java.lang.Class[]) methodType.

        Arguments
        - nrtype: a return parameter type to replace the old one with

        Returns
        - the same type, except with the return type change

        Raises
        - NullPointerException: if `nrtype` is null
        """
        ...


    def hasPrimitives(self) -> bool:
        """
        Reports if this type contains a primitive argument or return value.
        The return type `void` counts as a primitive.

        Returns
        - True if any of the types are primitives
        """
        ...


    def hasWrappers(self) -> bool:
        """
        Reports if this type contains a wrapper argument or return value.
        Wrappers are types which box primitive values, such as Integer.
        The reference type `java.lang.Void` counts as a wrapper,
        if it occurs as a return type.

        Returns
        - True if any of the types are wrappers
        """
        ...


    def erase(self) -> "MethodType":
        """
        Erases all reference types to `Object`.
        Convenience method for .methodType(java.lang.Class, java.lang.Class[]) methodType.
        All primitive types (including `void`) will remain unchanged.

        Returns
        - a version of the original type with all reference types replaced
        """
        ...


    def generic(self) -> "MethodType":
        """
        Converts all types, both reference and primitive, to `Object`.
        Convenience method for .genericMethodType(int) genericMethodType.
        The expression `type.wrap().erase()` produces the same value
        as `type.generic()`.

        Returns
        - a version of the original type with all types replaced
        """
        ...


    def wrap(self) -> "MethodType":
        """
        Converts all primitive types to their corresponding wrapper types.
        Convenience method for .methodType(java.lang.Class, java.lang.Class[]) methodType.
        All reference types (including wrapper types) will remain unchanged.
        A `void` return type is changed to the type `java.lang.Void`.
        The expression `type.wrap().erase()` produces the same value
        as `type.generic()`.

        Returns
        - a version of the original type with all primitive types replaced
        """
        ...


    def unwrap(self) -> "MethodType":
        """
        Converts all wrapper types to their corresponding primitive types.
        Convenience method for .methodType(java.lang.Class, java.lang.Class[]) methodType.
        All primitive types (including `void`) will remain unchanged.
        A return type of `java.lang.Void` is changed to `void`.

        Returns
        - a version of the original type with all wrapper types replaced
        """
        ...


    def parameterType(self, num: int) -> type[Any]:
        """
        Returns the parameter type at the specified index, within this method type.

        Arguments
        - num: the index (zero-based) of the desired parameter type

        Returns
        - the selected parameter type

        Raises
        - IndexOutOfBoundsException: if `num` is not a valid index into `parameterArray()`
        """
        ...


    def parameterCount(self) -> int:
        """
        Returns the number of parameter types in this method type.

        Returns
        - the number of parameter types
        """
        ...


    def returnType(self) -> type[Any]:
        """
        Returns the return type of this method type.

        Returns
        - the return type
        """
        ...


    def parameterList(self) -> list[type[Any]]:
        """
        Presents the parameter types as a list (a convenience method).
        The list will be immutable.

        Returns
        - the parameter types (as an immutable list)
        """
        ...


    def lastParameterType(self) -> type[Any]:
        """
        Returns the last parameter type of this method type.
        If this type has no parameters, the sentinel value
        `void.class` is returned instead.

        Returns
        - the last parameter type if any, else `void.class`

        Since
        - 10

        Unknown Tags
        - 
        The sentinel value is chosen so that reflective queries can be
        made directly against the result value.
        The sentinel value cannot be confused with a real parameter,
        since `void` is never acceptable as a parameter type.
        For variable arity invocation modes, the expression
        Class.getComponentType lastParameterType().getComponentType()
        is useful to query the type of the "varargs" parameter.
        """
        ...


    def parameterArray(self) -> list[type[Any]]:
        """
        Presents the parameter types as an array (a convenience method).
        Changes to the array will not result in changes to the type.

        Returns
        - the parameter types (as a fresh copy if necessary)
        """
        ...


    def equals(self, x: "Object") -> bool:
        ...


    def hashCode(self) -> int:
        """
        Returns the hash code value for this method type.
        It is defined to be the same as the hashcode of a List
        whose elements are the return type followed by the
        parameter types.

        Returns
        - the hash code value for this method type

        See
        - List.hashCode()
        """
        ...


    def toString(self) -> str:
        """
        Returns a string representation of the method type,
        of the form `"(PT0,PT1...)RT"`.
        The string representation of a method type is a
        parenthesis enclosed, comma separated list of type names,
        followed immediately by the return type.
        
        Each type is represented by its
        java.lang.Class.getSimpleName simple name.
        """
        ...


    @staticmethod
    def fromMethodDescriptorString(descriptor: str, loader: "ClassLoader") -> "MethodType":
        """
        Finds or creates an instance of a method type, given the spelling of its bytecode descriptor.
        Convenience method for .methodType(java.lang.Class, java.lang.Class[]) methodType.
        Any class or interface name embedded in the descriptor string will be
        resolved by the given loader (or if it is null, on the system class loader).
        
        Note that it is possible to encounter method types which cannot be
        constructed by this method, because their component types are
        not all reachable from a common class loader.
        
        This method is included for the benefit of applications that must
        generate bytecodes that process method handles and `invokedynamic`.

        Arguments
        - descriptor: a bytecode-level type descriptor string "(T...)T"
        - loader: the class loader in which to look up the types

        Returns
        - a method type matching the bytecode-level type descriptor

        Raises
        - NullPointerException: if the string is null
        - IllegalArgumentException: if the string is not well-formed
        - TypeNotPresentException: if a named type cannot be found
        - SecurityException: if the security manager is present and
                `loader` is `null` and the caller does not have the
                RuntimePermission`("getClassLoader")`
        """
        ...


    def toMethodDescriptorString(self) -> str:
        """
        Returns a descriptor string for the method type.  This method
        is equivalent to calling .descriptorString() MethodType::descriptorString.
        
        
        Note that this is not a strict inverse of .fromMethodDescriptorString fromMethodDescriptorString.
        Two distinct classes which share a common name but have different class loaders
        will appear identical when viewed within descriptor strings.
        
        This method is included for the benefit of applications that must
        generate bytecodes that process method handles and `invokedynamic`.
        .fromMethodDescriptorString(java.lang.String, java.lang.ClassLoader) fromMethodDescriptorString,
        because the latter requires a suitable class loader argument.

        Returns
        - the descriptor string for this method type

        See
        - <a href=".descriptor">Nominal Descriptor for `MethodType`</a>

        Unknown Tags
        - 4.3.3 Method Descriptors
        """
        ...


    def descriptorString(self) -> str:
        """
        Returns a descriptor string for this method type.
        
        
        If this method type can be <a href="#descriptor">described nominally</a>,
        then the result is a method type descriptor (JVMS 4.3.3).
        MethodTypeDesc MethodTypeDesc for this method type
        can be produced by calling MethodTypeDesc.ofDescriptor(String)
        MethodTypeDesc::ofDescriptor with the result descriptor string.
        
        If this method type cannot be <a href="#descriptor">described nominally</a>
        and the result is a string of the form:
        <blockquote>`"(<parameter-descriptors>)<return-descriptor>"`</blockquote>
        where `<parameter-descriptors>` is the concatenation of the
        Class.descriptorString() descriptor string of all
        of the parameter types and the Class.descriptorString() descriptor string
        of the return type. No java.lang.constant.MethodTypeDesc MethodTypeDesc
        can be produced from the result string.

        Returns
        - the descriptor string for this method type

        See
        - <a href=".descriptor">Nominal Descriptor for `MethodType`</a>

        Since
        - 12

        Unknown Tags
        - 4.3.3 Method Descriptors
        """
        ...


    def describeConstable(self) -> "Optional"["MethodTypeDesc"]:
        """
        Returns a nominal descriptor for this instance, if one can be
        constructed, or an empty Optional if one cannot be.

        Returns
        - An Optional containing the resulting nominal descriptor,
        or an empty Optional if one cannot be constructed.

        See
        - <a href=".descriptor">Nominal Descriptor for `MethodType`</a>

        Since
        - 12
        """
        ...



"""
Python module generated from Java source file java.util.AbstractCollection

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import *
from jdk.internal.util import ArraysSupport
from typing import Any, Callable, Iterable, Tuple


class AbstractCollection(Collection):

    def iterator(self) -> Iterator["E"]:
        """
        Returns an iterator over the elements contained in this collection.

        Returns
        - an iterator over the elements contained in this collection
        """
        ...


    def size(self) -> int:
        ...


    def isEmpty(self) -> bool:
        """
        Unknown Tags
        - This implementation returns `size() == 0`.
        """
        ...


    def contains(self, o: "Object") -> bool:
        """
        Raises
        - ClassCastException: 
        - NullPointerException: 

        Unknown Tags
        - This implementation iterates over the elements in the collection,
        checking each element in turn for equality with the specified element.
        """
        ...


    def toArray(self) -> list["Object"]:
        """
        Unknown Tags
        - This implementation returns an array containing all the elements
        returned by this collection's iterator, in the same order, stored in
        consecutive elements of the array, starting with index `0`.
        The length of the returned array is equal to the number of elements
        returned by the iterator, even if the size of this collection changes
        during iteration, as might happen if the collection permits
        concurrent modification during iteration.  The `size` method is
        called only as an optimization hint; the correct result is returned
        even if the iterator returns a different number of elements.
        
        This method is equivalent to:
        
         ``` `List<E> list = new ArrayList<E>(size());
        for (E e : this)
            list.add(e);
        return list.toArray();````
        """
        ...


    def toArray(self, a: list["T"]) -> list["T"]:
        """
        Raises
        - ArrayStoreException: 
        - NullPointerException: 

        Unknown Tags
        - This implementation returns an array containing all the elements
        returned by this collection's iterator in the same order, stored in
        consecutive elements of the array, starting with index `0`.
        If the number of elements returned by the iterator is too large to
        fit into the specified array, then the elements are returned in a
        newly allocated array with length equal to the number of elements
        returned by the iterator, even if the size of this collection
        changes during iteration, as might happen if the collection permits
        concurrent modification during iteration.  The `size` method is
        called only as an optimization hint; the correct result is returned
        even if the iterator returns a different number of elements.
        
        This method is equivalent to:
        
         ``` `List<E> list = new ArrayList<E>(size());
        for (E e : this)
            list.add(e);
        return list.toArray(a);````
        """
        ...


    def add(self, e: "E") -> bool:
        """
        Raises
        - UnsupportedOperationException: 
        - ClassCastException: 
        - NullPointerException: 
        - IllegalArgumentException: 
        - IllegalStateException: 

        Unknown Tags
        - This implementation always throws an
        `UnsupportedOperationException`.
        """
        ...


    def remove(self, o: "Object") -> bool:
        """
        Raises
        - UnsupportedOperationException: 
        - ClassCastException: 
        - NullPointerException: 

        Unknown Tags
        - This implementation iterates over the collection looking for the
        specified element.  If it finds the element, it removes the element
        from the collection using the iterator's remove method.
        
        Note that this implementation throws an
        `UnsupportedOperationException` if the iterator returned by this
        collection's iterator method does not implement the `remove`
        method and this collection contains the specified object.
        """
        ...


    def containsAll(self, c: Iterable[Any]) -> bool:
        """
        Raises
        - ClassCastException: 
        - NullPointerException: 

        See
        - .contains(Object)

        Unknown Tags
        - This implementation iterates over the specified collection,
        checking each element returned by the iterator in turn to see
        if it's contained in this collection.  If all elements are so
        contained `True` is returned, otherwise `False`.
        """
        ...


    def addAll(self, c: Iterable["E"]) -> bool:
        """
        Raises
        - UnsupportedOperationException: 
        - ClassCastException: 
        - NullPointerException: 
        - IllegalArgumentException: 
        - IllegalStateException: 

        See
        - .add(Object)

        Unknown Tags
        - This implementation iterates over the specified collection, and adds
        each object returned by the iterator to this collection, in turn.
        
        Note that this implementation will throw an
        `UnsupportedOperationException` unless `add` is
        overridden (assuming the specified collection is non-empty).
        """
        ...


    def removeAll(self, c: Iterable[Any]) -> bool:
        """
        Raises
        - UnsupportedOperationException: 
        - ClassCastException: 
        - NullPointerException: 

        See
        - .contains(Object)

        Unknown Tags
        - This implementation iterates over this collection, checking each
        element returned by the iterator in turn to see if it's contained
        in the specified collection.  If it's so contained, it's removed from
        this collection with the iterator's `remove` method.
        
        Note that this implementation will throw an
        `UnsupportedOperationException` if the iterator returned by the
        `iterator` method does not implement the `remove` method
        and this collection contains one or more elements in common with the
        specified collection.
        """
        ...


    def retainAll(self, c: Iterable[Any]) -> bool:
        """
        Raises
        - UnsupportedOperationException: 
        - ClassCastException: 
        - NullPointerException: 

        See
        - .contains(Object)

        Unknown Tags
        - This implementation iterates over this collection, checking each
        element returned by the iterator in turn to see if it's contained
        in the specified collection.  If it's not so contained, it's removed
        from this collection with the iterator's `remove` method.
        
        Note that this implementation will throw an
        `UnsupportedOperationException` if the iterator returned by the
        `iterator` method does not implement the `remove` method
        and this collection contains one or more elements not present in the
        specified collection.
        """
        ...


    def clear(self) -> None:
        """
        Raises
        - UnsupportedOperationException: 

        Unknown Tags
        - This implementation iterates over this collection, removing each
        element using the `Iterator.remove` operation.  Most
        implementations will probably choose to override this method for
        efficiency.
        
        Note that this implementation will throw an
        `UnsupportedOperationException` if the iterator returned by this
        collection's `iterator` method does not implement the
        `remove` method and this collection is non-empty.
        """
        ...


    def toString(self) -> str:
        """
        Returns a string representation of this collection.  The string
        representation consists of a list of the collection's elements in the
        order they are returned by its iterator, enclosed in square brackets
        (`"[]"`).  Adjacent elements are separated by the characters
        `", "` (comma and space).  Elements are converted to strings as
        by String.valueOf(Object).

        Returns
        - a string representation of this collection
        """
        ...



"""
Python module generated from Java source file org.bukkit.block.data.AnaloguePowerable

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block.data import *
from typing import Any, Callable, Iterable, Tuple


class AnaloguePowerable(BlockData):
    """
    'power' represents the redstone power level currently being emitted or
    transmitted via this block.
    
    May not be over 9000 or .getMaximumPower() (usually 15).
    """

    def getPower(self) -> int:
        """
        Gets the value of the 'power' property.

        Returns
        - the 'power' value
        """
        ...


    def setPower(self, power: int) -> None:
        """
        Sets the value of the 'power' property.

        Arguments
        - power: the new 'power' value
        """
        ...


    def getMaximumPower(self) -> int:
        """
        Gets the maximum allowed value of the 'power' property.

        Returns
        - the maximum 'power' value
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.player.PlayerTakeLecternBookEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import Lectern
from org.bukkit.entity import Player
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.player import *
from org.bukkit.inventory import ItemStack
from typing import Any, Callable, Iterable, Tuple


class PlayerTakeLecternBookEvent(PlayerEvent, Cancellable):
    """
    This event is called when a player clicks the button to take a book of a
    Lectern. If this event is cancelled the book remains on the lectern.
    """

    def __init__(self, who: "Player", lectern: "Lectern"):
        ...


    def getLectern(self) -> "Lectern":
        """
        Gets the lectern involved.

        Returns
        - the Lectern
        """
        ...


    def getBook(self) -> "ItemStack":
        """
        Gets the current ItemStack on the lectern.

        Returns
        - the ItemStack on the Lectern
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.bukkit.block.Furnace

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import *
from org.bukkit.inventory import CookingRecipe
from org.bukkit.inventory import FurnaceInventory
from typing import Any, Callable, Iterable, Tuple


class Furnace(Container):
    """
    Represents a captured state of a furnace.
    """

    def getBurnTime(self) -> int:
        """
        Get burn time.

        Returns
        - Burn time
        """
        ...


    def setBurnTime(self, burnTime: int) -> None:
        """
        Set burn time.
        
        A burn time greater than 0 will cause this block to be lit, whilst a time
        less than 0 will extinguish it.

        Arguments
        - burnTime: Burn time
        """
        ...


    def getCookTime(self) -> int:
        """
        Get cook time.
        
        This is the amount of time the item has been cooking for.

        Returns
        - Cook time
        """
        ...


    def setCookTime(self, cookTime: int) -> None:
        """
        Set cook time.
        
        This is the amount of time the item has been cooking for.

        Arguments
        - cookTime: Cook time
        """
        ...


    def getCookTimeTotal(self) -> int:
        """
        Get cook time total.
        
        This is the amount of time the item is required to cook for.

        Returns
        - Cook time total
        """
        ...


    def setCookTimeTotal(self, cookTimeTotal: int) -> None:
        """
        Set cook time.
        
        This is the amount of time the item is required to cook for.

        Arguments
        - cookTimeTotal: Cook time total
        """
        ...


    def getRecipesUsed(self) -> dict["CookingRecipe"[Any], "Integer"]:
        """
        Get the recipes used in this furnace.
        
        **Note:** These recipes used are reset when the result item is
        manually taken from the furnace.

        Returns
        - An immutable map with the recipes used and the times used
        """
        ...


    def getInventory(self) -> "FurnaceInventory":
        ...


    def getSnapshotInventory(self) -> "FurnaceInventory":
        ...



from .NullConversationPrefix import NullConversationPrefix
from .InactivityConversationCanceller import InactivityConversationCanceller
from .StringPrompt import StringPrompt
from .ConversationAbandonedEvent import ConversationAbandonedEvent
from .ValidatingPrompt import ValidatingPrompt
from .ConversationContext import ConversationContext
from .ConversationFactory import ConversationFactory
from .PlayerNamePrompt import PlayerNamePrompt
from .ConversationCanceller import ConversationCanceller
from .NumericPrompt import NumericPrompt
from .FixedSetPrompt import FixedSetPrompt
from .BooleanPrompt import BooleanPrompt
from .PluginNameConversationPrefix import PluginNameConversationPrefix
from .Prompt import Prompt
from .RegexPrompt import RegexPrompt
from .Conversable import Conversable
from .MessagePrompt import MessagePrompt
from .ManuallyAbandonedConversationCanceller import ManuallyAbandonedConversationCanceller
from .ExactMatchConversationCanceller import ExactMatchConversationCanceller
from .Conversation import Conversation
from .ConversationPrefix import ConversationPrefix
from .ConversationAbandonedListener import ConversationAbandonedListener


"""
Python module generated from Java source file com.google.common.collect.Iterators

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.base import Function
from com.google.common.base import Objects
from com.google.common.base import Optional
from com.google.common.base import Preconditions
from com.google.common.base import Predicate
from com.google.common.collect import *
from com.google.common.primitives import Ints
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.util import ArrayDeque
from java.util import Arrays
from java.util import Collections
from java.util import Comparator
from java.util import Deque
from java.util import Enumeration
from java.util import Iterator
from java.util import NoSuchElementException
from java.util import PriorityQueue
from java.util import Queue
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import NonNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class Iterators:
    """
    This class contains static utility methods that operate on or return objects of type Iterator. Except as noted, each method has a corresponding Iterable-based method in the
    Iterables class.
    
    *Performance notes:* Unless otherwise noted, all of the iterators produced in this class
    are *lazy*, which means that they only advance the backing iteration when absolutely
    necessary.
    
    See the Guava User Guide section on <a href=
    "https://github.com/google/guava/wiki/CollectionUtilitiesExplained#iterables">`Iterators`</a>.

    Author(s)
    - Jared Levy

    Since
    - 2.0
    """

    @staticmethod
    def unmodifiableIterator(iterator: Iterator["T"]) -> "UnmodifiableIterator"["T"]:
        """
        Returns an unmodifiable view of `iterator`.
        """
        ...


    @staticmethod
    def unmodifiableIterator(iterator: "UnmodifiableIterator"["T"]) -> "UnmodifiableIterator"["T"]:
        """
        Simply returns its argument.

        Since
        - 10.0

        Deprecated
        - no need to use this
        """
        ...


    @staticmethod
    def size(iterator: Iterator[Any]) -> int:
        """
        Returns the number of elements remaining in `iterator`. The iterator will be left
        exhausted: its `hasNext()` method will return `False`.
        """
        ...


    @staticmethod
    def contains(iterator: Iterator[Any], element: "Object") -> bool:
        """
        Returns `True` if `iterator` contains `element`.
        """
        ...


    @staticmethod
    def removeAll(removeFrom: Iterator[Any], elementsToRemove: Iterable[Any]) -> bool:
        """
        Traverses an iterator and removes every element that belongs to the provided collection. The
        iterator will be left exhausted: its `hasNext()` method will return `False`.

        Arguments
        - removeFrom: the iterator to (potentially) remove elements from
        - elementsToRemove: the elements to remove

        Returns
        - `True` if any element was removed from `iterator`
        """
        ...


    @staticmethod
    def removeIf(removeFrom: Iterator["T"], predicate: "Predicate"["T"]) -> bool:
        """
        Removes every element that satisfies the provided predicate from the iterator. The iterator
        will be left exhausted: its `hasNext()` method will return `False`.

        Arguments
        - removeFrom: the iterator to (potentially) remove elements from
        - predicate: a predicate that determines whether an element should be removed

        Returns
        - `True` if any elements were removed from the iterator

        Since
        - 2.0
        """
        ...


    @staticmethod
    def retainAll(removeFrom: Iterator[Any], elementsToRetain: Iterable[Any]) -> bool:
        """
        Traverses an iterator and removes every element that does not belong to the provided
        collection. The iterator will be left exhausted: its `hasNext()` method will return
        `False`.

        Arguments
        - removeFrom: the iterator to (potentially) remove elements from
        - elementsToRetain: the elements to retain

        Returns
        - `True` if any element was removed from `iterator`
        """
        ...


    @staticmethod
    def elementsEqual(iterator1: Iterator[Any], iterator2: Iterator[Any]) -> bool:
        """
        Determines whether two iterators contain equal elements in the same order. More specifically,
        this method returns `True` if `iterator1` and `iterator2` contain the same
        number of elements and every element of `iterator1` is equal to the corresponding element
        of `iterator2`.
        
        Note that this will modify the supplied iterators, since they will have been advanced some
        number of elements forward.
        """
        ...


    @staticmethod
    def toString(iterator: Iterator[Any]) -> str:
        """
        Returns a string representation of `iterator`, with the format `[e1, e2, ..., en]`.
        The iterator will be left exhausted: its `hasNext()` method will return `False`.
        """
        ...


    @staticmethod
    def getOnlyElement(iterator: Iterator["T"]) -> "T":
        """
        Returns the single element contained in `iterator`.

        Raises
        - NoSuchElementException: if the iterator is empty
        - IllegalArgumentException: if the iterator contains multiple elements. The state of the
            iterator is unspecified.
        """
        ...


    @staticmethod
    def getOnlyElement(iterator: Iterator["T"], defaultValue: "T") -> "T":
        """
        Returns the single element contained in `iterator`, or `defaultValue` if the
        iterator is empty.

        Raises
        - IllegalArgumentException: if the iterator contains multiple elements. The state of the
            iterator is unspecified.
        """
        ...


    @staticmethod
    def toArray(iterator: Iterator["T"], type: type["T"]) -> list["T"]:
        """
        Copies an iterator's elements into an array. The iterator will be left exhausted: its `hasNext()` method will return `False`.

        Arguments
        - iterator: the iterator to copy
        - type: the type of the elements

        Returns
        - a newly-allocated array into which all the elements of the iterator have been copied
        """
        ...


    @staticmethod
    def addAll(addTo: Iterable["T"], iterator: Iterator["T"]) -> bool:
        """
        Adds all elements in `iterator` to `collection`. The iterator will be left
        exhausted: its `hasNext()` method will return `False`.

        Returns
        - `True` if `collection` was modified as a result of this operation
        """
        ...


    @staticmethod
    def frequency(iterator: Iterator[Any], element: "Object") -> int:
        """
        Returns the number of elements in the specified iterator that equal the specified object. The
        iterator will be left exhausted: its `hasNext()` method will return `False`.

        See
        - Collections.frequency
        """
        ...


    @staticmethod
    def cycle(iterable: Iterable["T"]) -> Iterator["T"]:
        """
        Returns an iterator that cycles indefinitely over the elements of `iterable`.
        
        The returned iterator supports `remove()` if the provided iterator does. After `remove()` is called, subsequent cycles omit the removed element, which is no longer in `iterable`. The iterator's `hasNext()` method returns `True` until `iterable`
        is empty.
        
        **Warning:** Typical uses of the resulting iterator may produce an infinite loop. You
        should use an explicit `break` or be certain that you will eventually remove all the
        elements.
        """
        ...


    @staticmethod
    def cycle(*elements: Tuple["T", ...]) -> Iterator["T"]:
        """
        Returns an iterator that cycles indefinitely over the provided elements.
        
        The returned iterator supports `remove()`. After `remove()` is called,
        subsequent cycles omit the removed element, but `elements` does not change. The
        iterator's `hasNext()` method returns `True` until all of the original elements
        have been removed.
        
        **Warning:** Typical uses of the resulting iterator may produce an infinite loop. You
        should use an explicit `break` or be certain that you will eventually remove all the
        elements.
        """
        ...


    @staticmethod
    def concat(a: Iterator["T"], b: Iterator["T"]) -> Iterator["T"]:
        """
        Combines two iterators into a single iterator. The returned iterator iterates across the
        elements in `a`, followed by the elements in `b`. The source iterators are not
        polled until necessary.
        
        The returned iterator supports `remove()` when the corresponding input iterator
        supports it.
        """
        ...


    @staticmethod
    def concat(a: Iterator["T"], b: Iterator["T"], c: Iterator["T"]) -> Iterator["T"]:
        """
        Combines three iterators into a single iterator. The returned iterator iterates across the
        elements in `a`, followed by the elements in `b`, followed by the elements in
        `c`. The source iterators are not polled until necessary.
        
        The returned iterator supports `remove()` when the corresponding input iterator
        supports it.
        """
        ...


    @staticmethod
    def concat(a: Iterator["T"], b: Iterator["T"], c: Iterator["T"], d: Iterator["T"]) -> Iterator["T"]:
        """
        Combines four iterators into a single iterator. The returned iterator iterates across the
        elements in `a`, followed by the elements in `b`, followed by the elements in
        `c`, followed by the elements in `d`. The source iterators are not polled until
        necessary.
        
        The returned iterator supports `remove()` when the corresponding input iterator
        supports it.
        """
        ...


    @staticmethod
    def concat(*inputs: Tuple[Iterator["T"], ...]) -> Iterator["T"]:
        """
        Combines multiple iterators into a single iterator. The returned iterator iterates across the
        elements of each iterator in `inputs`. The input iterators are not polled until
        necessary.
        
        The returned iterator supports `remove()` when the corresponding input iterator
        supports it.

        Raises
        - NullPointerException: if any of the provided iterators is null
        """
        ...


    @staticmethod
    def concat(inputs: Iterator[Iterator["T"]]) -> Iterator["T"]:
        """
        Combines multiple iterators into a single iterator. The returned iterator iterates across the
        elements of each iterator in `inputs`. The input iterators are not polled until
        necessary.
        
        The returned iterator supports `remove()` when the corresponding input iterator
        supports it. The methods of the returned iterator may throw `NullPointerException` if any
        of the input iterators is null.
        """
        ...


    @staticmethod
    def partition(iterator: Iterator["T"], size: int) -> "UnmodifiableIterator"[list["T"]]:
        """
        Divides an iterator into unmodifiable sublists of the given size (the final list may be
        smaller). For example, partitioning an iterator containing `[a, b, c, d, e]` with a
        partition size of 3 yields `[[a, b, c], [d, e]]` -- an outer iterator containing two
        inner lists of three and two elements, all in the original order.
        
        The returned lists implement java.util.RandomAccess.
        
        **Note:** The current implementation eagerly allocates storage for `size` elements.
        As a consequence, passing values like `Integer.MAX_VALUE` can lead to OutOfMemoryError.

        Arguments
        - iterator: the iterator to return a partitioned view of
        - size: the desired size of each partition (the last may be smaller)

        Returns
        - an iterator of immutable lists containing the elements of `iterator` divided into
            partitions

        Raises
        - IllegalArgumentException: if `size` is nonpositive
        """
        ...


    @staticmethod
    def paddedPartition(iterator: Iterator["T"], size: int) -> "UnmodifiableIterator"[list["T"]]:
        """
        Divides an iterator into unmodifiable sublists of the given size, padding the final iterator
        with null values if necessary. For example, partitioning an iterator containing `[a, b,
        c, d, e]` with a partition size of 3 yields `[[a, b, c], [d, e, null]]` -- an outer
        iterator containing two inner lists of three elements each, all in the original order.
        
        The returned lists implement java.util.RandomAccess.

        Arguments
        - iterator: the iterator to return a partitioned view of
        - size: the desired size of each partition

        Returns
        - an iterator of immutable lists containing the elements of `iterator` divided into
            partitions (the final iterable may have trailing null elements)

        Raises
        - IllegalArgumentException: if `size` is nonpositive
        """
        ...


    @staticmethod
    def filter(unfiltered: Iterator["T"], retainIfTrue: "Predicate"["T"]) -> "UnmodifiableIterator"["T"]:
        """
        Returns a view of `unfiltered` containing all elements that satisfy the input predicate
        `retainIfTrue`.
        """
        ...


    @staticmethod
    def filter(unfiltered: Iterator[Any], desiredType: type["T"]) -> "UnmodifiableIterator"["T"]:
        """
        Returns a view of `unfiltered` containing all elements that are of the type `desiredType`.
        """
        ...


    @staticmethod
    def any(iterator: Iterator["T"], predicate: "Predicate"["T"]) -> bool:
        """
        Returns `True` if one or more elements returned by `iterator` satisfy the given
        predicate.
        """
        ...


    @staticmethod
    def all(iterator: Iterator["T"], predicate: "Predicate"["T"]) -> bool:
        """
        Returns `True` if every element returned by `iterator` satisfies the given
        predicate. If `iterator` is empty, `True` is returned.
        """
        ...


    @staticmethod
    def find(iterator: Iterator["T"], predicate: "Predicate"["T"]) -> "T":
        """
        Returns the first element in `iterator` that satisfies the given predicate; use this
        method only when such an element is known to exist. If no such element is found, the iterator
        will be left exhausted: its `hasNext()` method will return `False`. If it is
        possible that *no* element will match, use .tryFind or .find(Iterator,
        Predicate, Object) instead.

        Raises
        - NoSuchElementException: if no element in `iterator` matches the given predicate
        """
        ...


    @staticmethod
    def find(iterator: Iterator["T"], predicate: "Predicate"["T"], defaultValue: "T") -> "T":
        ...


    @staticmethod
    def tryFind(iterator: Iterator["T"], predicate: "Predicate"["T"]) -> "Optional"["T"]:
        """
        Returns an Optional containing the first element in `iterator` that satisfies the
        given predicate, if such an element exists. If no such element is found, an empty Optional will be returned from this method and the iterator will be left exhausted: its `hasNext()` method will return `False`.
        
        **Warning:** avoid using a `predicate` that matches `null`. If `null`
        is matched in `iterator`, a NullPointerException will be thrown.

        Since
        - 11.0
        """
        ...


    @staticmethod
    def indexOf(iterator: Iterator["T"], predicate: "Predicate"["T"]) -> int:
        """
        Returns the index in `iterator` of the first element that satisfies the provided `predicate`, or `-1` if the Iterator has no such elements.
        
        More formally, returns the lowest index `i` such that `predicate.apply(Iterators.get(iterator, i))` returns `True`, or `-1` if there is no
        such index.
        
        If -1 is returned, the iterator will be left exhausted: its `hasNext()` method will
        return `False`. Otherwise, the iterator will be set to the element which satisfies the
        `predicate`.

        Since
        - 2.0
        """
        ...


    @staticmethod
    def transform(fromIterator: Iterator["F"], function: "Function"["F", "T"]) -> Iterator["T"]:
        """
        Returns a view containing the result of applying `function` to each element of `fromIterator`.
        
        The returned iterator supports `remove()` if `fromIterator` does. After a
        successful `remove()` call, `fromIterator` no longer contains the corresponding
        element.
        """
        ...


    @staticmethod
    def get(iterator: Iterator["T"], position: int) -> "T":
        """
        Advances `iterator` `position + 1` times, returning the element at the `position`th position.

        Arguments
        - position: position of the element to return

        Returns
        - the element at the specified position in `iterator`

        Raises
        - IndexOutOfBoundsException: if `position` is negative or greater than or equal to
            the number of elements remaining in `iterator`
        """
        ...


    @staticmethod
    def get(iterator: Iterator["T"], position: int, defaultValue: "T") -> "T":
        """
        Advances `iterator` `position + 1` times, returning the element at the `position`th position or `defaultValue` otherwise.

        Arguments
        - position: position of the element to return
        - defaultValue: the default value to return if the iterator is empty or if `position`
            is greater than the number of elements remaining in `iterator`

        Returns
        - the element at the specified position in `iterator` or `defaultValue` if
            `iterator` produces fewer than `position + 1` elements.

        Raises
        - IndexOutOfBoundsException: if `position` is negative

        Since
        - 4.0
        """
        ...


    @staticmethod
    def getNext(iterator: Iterator["T"], defaultValue: "T") -> "T":
        """
        Returns the next element in `iterator` or `defaultValue` if the iterator is empty.
        The Iterables analog to this method is Iterables.getFirst.

        Arguments
        - defaultValue: the default value to return if the iterator is empty

        Returns
        - the next element of `iterator` or the default value

        Since
        - 7.0
        """
        ...


    @staticmethod
    def getLast(iterator: Iterator["T"]) -> "T":
        """
        Advances `iterator` to the end, returning the last element.

        Returns
        - the last element of `iterator`

        Raises
        - NoSuchElementException: if the iterator is empty
        """
        ...


    @staticmethod
    def getLast(iterator: Iterator["T"], defaultValue: "T") -> "T":
        """
        Advances `iterator` to the end, returning the last element or `defaultValue` if the
        iterator is empty.

        Arguments
        - defaultValue: the default value to return if the iterator is empty

        Returns
        - the last element of `iterator`

        Since
        - 3.0
        """
        ...


    @staticmethod
    def advance(iterator: Iterator[Any], numberToAdvance: int) -> int:
        """
        Calls `next()` on `iterator`, either `numberToAdvance` times or until `hasNext()` returns `False`, whichever comes first.

        Returns
        - the number of elements the iterator was advanced

        Since
        - 13.0 (since 3.0 as `Iterators.skip`)
        """
        ...


    @staticmethod
    def limit(iterator: Iterator["T"], limitSize: int) -> Iterator["T"]:
        """
        Returns a view containing the first `limitSize` elements of `iterator`. If `iterator` contains fewer than `limitSize` elements, the returned view contains all of its
        elements. The returned iterator supports `remove()` if `iterator` does.

        Arguments
        - iterator: the iterator to limit
        - limitSize: the maximum number of elements in the returned iterator

        Raises
        - IllegalArgumentException: if `limitSize` is negative

        Since
        - 3.0
        """
        ...


    @staticmethod
    def consumingIterator(iterator: Iterator["T"]) -> Iterator["T"]:
        """
        Returns a view of the supplied `iterator` that removes each element from the supplied
        `iterator` as it is returned.
        
        The provided iterator must support Iterator.remove() or else the returned iterator
        will fail on the first call to `next`. The returned Iterator is also not
        thread-safe.

        Arguments
        - iterator: the iterator to remove and return elements from

        Returns
        - an iterator that removes and returns elements from the supplied iterator

        Since
        - 2.0
        """
        ...


    @staticmethod
    def forArray(*array: Tuple["T", ...]) -> "UnmodifiableIterator"["T"]:
        """
        Returns an iterator containing the elements of `array` in order. The returned iterator is
        a view of the array; subsequent changes to the array will be reflected in the iterator.
        
        **Note:** It is often preferable to represent your data using a collection type, for
        example using Arrays.asList(Object[]), making this method unnecessary.
        
        The `Iterable` equivalent of this method is either Arrays.asList(Object[]),
        ImmutableList.copyOf(Object[])}, or ImmutableList.of.
        """
        ...


    @staticmethod
    def singletonIterator(value: "T") -> "UnmodifiableIterator"["T"]:
        """
        Returns an iterator containing only `value`.
        
        The Iterable equivalent of this method is Collections.singleton.
        """
        ...


    @staticmethod
    def forEnumeration(enumeration: "Enumeration"["T"]) -> "UnmodifiableIterator"["T"]:
        """
        Adapts an `Enumeration` to the `Iterator` interface.
        
        This method has no equivalent in Iterables because viewing an `Enumeration` as
        an `Iterable` is impossible. However, the contents can be *copied* into a collection
        using Collections.list.
        
        **Java 9 users:** use `enumeration.asIterator()` instead, unless it is important to
        return an `UnmodifiableIterator` instead of a plain `Iterator`.
        """
        ...


    @staticmethod
    def asEnumeration(iterator: Iterator["T"]) -> "Enumeration"["T"]:
        """
        Adapts an `Iterator` to the `Enumeration` interface.
        
        The `Iterable` equivalent of this method is either Collections.enumeration (if
        you have a Collection), or `Iterators.asEnumeration(collection.iterator())`.
        """
        ...


    @staticmethod
    def peekingIterator(iterator: Iterator["T"]) -> "PeekingIterator"["T"]:
        """
        Returns a `PeekingIterator` backed by the given iterator.
        
        Calls to the `peek` method with no intervening calls to `next` do not affect the
        iteration, and hence return the same object each time. A subsequent call to `next` is
        guaranteed to return the same object again. For example:
        
        ````PeekingIterator<String> peekingIterator =
            Iterators.peekingIterator(Iterators.forArray("a", "b"));
        String a1 = peekingIterator.peek(); // returns "a"
        String a2 = peekingIterator.peek(); // also returns "a"
        String a3 = peekingIterator.next(); // also returns "a"````
        
        Any structural changes to the underlying iteration (aside from those performed by the
        iterator's own PeekingIterator.remove() method) will leave the iterator in an undefined
        state.
        
        The returned iterator does not support removal after peeking, as explained by PeekingIterator.remove().
        
        Note: If the given iterator is already a `PeekingIterator`, it *might* be
        returned to the caller, although this is neither guaranteed to occur nor required to be
        consistent. For example, this method *might* choose to pass through recognized
        implementations of `PeekingIterator` when the behavior of the implementation is known to
        meet the contract guaranteed by this method.
        
        There is no Iterable equivalent to this method, so use this method to wrap each
        individual iterator as it is generated.

        Arguments
        - iterator: the backing iterator. The PeekingIterator assumes ownership of this
            iterator, so users should cease making direct calls to it after calling this method.

        Returns
        - a peeking iterator backed by that iterator. Apart from the additional PeekingIterator.peek() method, this iterator behaves exactly the same as `iterator`.
        """
        ...


    @staticmethod
    def peekingIterator(iterator: "PeekingIterator"["T"]) -> "PeekingIterator"["T"]:
        """
        Simply returns its argument.

        Since
        - 10.0

        Deprecated
        - no need to use this
        """
        ...


    @staticmethod
    def mergeSorted(iterators: Iterable[Iterator["T"]], comparator: "Comparator"["T"]) -> "UnmodifiableIterator"["T"]:
        """
        Returns an iterator over the merged contents of all given `iterators`, traversing every
        element of the input iterators. Equivalent entries will not be de-duplicated.
        
        Callers must ensure that the source `iterators` are in non-descending order as this
        method does not sort its input.
        
        For any equivalent elements across all `iterators`, it is undefined which element is
        returned first.

        Since
        - 11.0
        """
        ...



"""
Python module generated from Java source file java.util.concurrent.locks.Lock

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util.concurrent import TimeUnit
from java.util.concurrent.locks import *
from typing import Any, Callable, Iterable, Tuple


class Lock:
    """
    `Lock` implementations provide more extensive locking
    operations than can be obtained using `synchronized` methods
    and statements.  They allow more flexible structuring, may have
    quite different properties, and may support multiple associated
    Condition objects.
    
    A lock is a tool for controlling access to a shared resource by
    multiple threads. Commonly, a lock provides exclusive access to a
    shared resource: only one thread at a time can acquire the lock and
    all access to the shared resource requires that the lock be
    acquired first. However, some locks may allow concurrent access to
    a shared resource, such as the read lock of a ReadWriteLock.
    
    The use of `synchronized` methods or statements provides
    access to the implicit monitor lock associated with every object, but
    forces all lock acquisition and release to occur in a block-structured way:
    when multiple locks are acquired they must be released in the opposite
    order, and all locks must be released in the same lexical scope in which
    they were acquired.
    
    While the scoping mechanism for `synchronized` methods
    and statements makes it much easier to program with monitor locks,
    and helps avoid many common programming errors involving locks,
    there are occasions where you need to work with locks in a more
    flexible way. For example, some algorithms for traversing
    concurrently accessed data structures require the use of
    &quot;hand-over-hand&quot; or &quot;chain locking&quot;: you
    acquire the lock of node A, then node B, then release A and acquire
    C, then release B and acquire D and so on.  Implementations of the
    `Lock` interface enable the use of such techniques by
    allowing a lock to be acquired and released in different scopes,
    and allowing multiple locks to be acquired and released in any
    order.
    
    With this increased flexibility comes additional
    responsibility. The absence of block-structured locking removes the
    automatic release of locks that occurs with `synchronized`
    methods and statements. In most cases, the following idiom
    should be used:
    
    ``` `Lock l = ...;
    l.lock();
    try {
      // access the resource protected by this lock` finally {
      l.unlock();
    }}```
    
    When locking and unlocking occur in different scopes, care must be
    taken to ensure that all code that is executed while the lock is
    held is protected by try-finally or try-catch to ensure that the
    lock is released when necessary.
    
    `Lock` implementations provide additional functionality
    over the use of `synchronized` methods and statements by
    providing a non-blocking attempt to acquire a lock (.tryLock()), an attempt to acquire the lock that can be
    interrupted (.lockInterruptibly, and an attempt to acquire
    the lock that can timeout (.tryLock(long, TimeUnit)).
    
    A `Lock` class can also provide behavior and semantics
    that is quite different from that of the implicit monitor lock,
    such as guaranteed ordering, non-reentrant usage, or deadlock
    detection. If an implementation provides such specialized semantics
    then the implementation must document those semantics.
    
    Note that `Lock` instances are just normal objects and can
    themselves be used as the target in a `synchronized` statement.
    Acquiring the
    monitor lock of a `Lock` instance has no specified relationship
    with invoking any of the .lock methods of that instance.
    It is recommended that to avoid confusion you never use `Lock`
    instances in this way, except within their own implementation.
    
    Except where noted, passing a `null` value for any
    parameter will result in a NullPointerException being
    thrown.
    
    <h2>Memory Synchronization</h2>
    
    All `Lock` implementations *must* enforce the same
    memory synchronization semantics as provided by the built-in monitor
    lock, as described in
    Chapter 17 of
    <cite>The Java Language Specification</cite>:
    
    - A successful `lock` operation has the same memory
    synchronization effects as a successful *Lock* action.
    - A successful `unlock` operation has the same
    memory synchronization effects as a successful *Unlock* action.
    
    
    Unsuccessful locking and unlocking operations, and reentrant
    locking/unlocking operations, do not require any memory
    synchronization effects.
    
    <h2>Implementation Considerations</h2>
    
    The three forms of lock acquisition (interruptible,
    non-interruptible, and timed) may differ in their performance
    characteristics, ordering guarantees, or other implementation
    qualities.  Further, the ability to interrupt the *ongoing*
    acquisition of a lock may not be available in a given `Lock`
    class.  Consequently, an implementation is not required to define
    exactly the same guarantees or semantics for all three forms of
    lock acquisition, nor is it required to support interruption of an
    ongoing lock acquisition.  An implementation is required to clearly
    document the semantics and guarantees provided by each of the
    locking methods. It must also obey the interruption semantics as
    defined in this interface, to the extent that interruption of lock
    acquisition is supported: which is either totally, or only on
    method entry.
    
    As interruption generally implies cancellation, and checks for
    interruption are often infrequent, an implementation can favor responding
    to an interrupt over normal method return. This is True even if it can be
    shown that the interrupt occurred after another action may have unblocked
    the thread. An implementation should document this behavior.

    Author(s)
    - Doug Lea

    See
    - ReadWriteLock

    Since
    - 1.5

    Unknown Tags
    - 17.4 Memory Model
    """

    def lock(self) -> None:
        """
        Acquires the lock.
        
        If the lock is not available then the current thread becomes
        disabled for thread scheduling purposes and lies dormant until the
        lock has been acquired.
        
        **Implementation Considerations**
        
        A `Lock` implementation may be able to detect erroneous use
        of the lock, such as an invocation that would cause deadlock, and
        may throw an (unchecked) exception in such circumstances.  The
        circumstances and the exception type must be documented by that
        `Lock` implementation.
        """
        ...


    def lockInterruptibly(self) -> None:
        """
        Acquires the lock unless the current thread is
        Thread.interrupt interrupted.
        
        Acquires the lock if it is available and returns immediately.
        
        If the lock is not available then the current thread becomes
        disabled for thread scheduling purposes and lies dormant until
        one of two things happens:
        
        
        - The lock is acquired by the current thread; or
        - Some other thread Thread.interrupt interrupts the
        current thread, and interruption of lock acquisition is supported.
        
        
        If the current thread:
        
        - has its interrupted status set on entry to this method; or
        - is Thread.interrupt interrupted while acquiring the
        lock, and interruption of lock acquisition is supported,
        
        then InterruptedException is thrown and the current thread's
        interrupted status is cleared.
        
        **Implementation Considerations**
        
        The ability to interrupt a lock acquisition in some
        implementations may not be possible, and if possible may be an
        expensive operation.  The programmer should be aware that this
        may be the case. An implementation should document when this is
        the case.
        
        An implementation can favor responding to an interrupt over
        normal method return.
        
        A `Lock` implementation may be able to detect
        erroneous use of the lock, such as an invocation that would
        cause deadlock, and may throw an (unchecked) exception in such
        circumstances.  The circumstances and the exception type must
        be documented by that `Lock` implementation.

        Raises
        - InterruptedException: if the current thread is
                interrupted while acquiring the lock (and interruption
                of lock acquisition is supported)
        """
        ...


    def tryLock(self) -> bool:
        """
        Acquires the lock only if it is free at the time of invocation.
        
        Acquires the lock if it is available and returns immediately
        with the value `True`.
        If the lock is not available then this method will return
        immediately with the value `False`.
        
        A typical usage idiom for this method would be:
        ``` `Lock lock = ...;
        if (lock.tryLock()) {
          try {
            // manipulate protected state` finally {
            lock.unlock();
          }
        } else {
          // perform alternative actions
        }}```
        
        This usage ensures that the lock is unlocked if it was acquired, and
        doesn't try to unlock if the lock was not acquired.

        Returns
        - `True` if the lock was acquired and
                `False` otherwise
        """
        ...


    def tryLock(self, time: int, unit: "TimeUnit") -> bool:
        """
        Acquires the lock if it is free within the given waiting time and the
        current thread has not been Thread.interrupt interrupted.
        
        If the lock is available this method returns immediately
        with the value `True`.
        If the lock is not available then
        the current thread becomes disabled for thread scheduling
        purposes and lies dormant until one of three things happens:
        
        - The lock is acquired by the current thread; or
        - Some other thread Thread.interrupt interrupts the
        current thread, and interruption of lock acquisition is supported; or
        - The specified waiting time elapses
        
        
        If the lock is acquired then the value `True` is returned.
        
        If the current thread:
        
        - has its interrupted status set on entry to this method; or
        - is Thread.interrupt interrupted while acquiring
        the lock, and interruption of lock acquisition is supported,
        
        then InterruptedException is thrown and the current thread's
        interrupted status is cleared.
        
        If the specified waiting time elapses then the value `False`
        is returned.
        If the time is
        less than or equal to zero, the method will not wait at all.
        
        **Implementation Considerations**
        
        The ability to interrupt a lock acquisition in some implementations
        may not be possible, and if possible may
        be an expensive operation.
        The programmer should be aware that this may be the case. An
        implementation should document when this is the case.
        
        An implementation can favor responding to an interrupt over normal
        method return, or reporting a timeout.
        
        A `Lock` implementation may be able to detect
        erroneous use of the lock, such as an invocation that would cause
        deadlock, and may throw an (unchecked) exception in such circumstances.
        The circumstances and the exception type must be documented by that
        `Lock` implementation.

        Arguments
        - time: the maximum time to wait for the lock
        - unit: the time unit of the `time` argument

        Returns
        - `True` if the lock was acquired and `False`
                if the waiting time elapsed before the lock was acquired

        Raises
        - InterruptedException: if the current thread is interrupted
                while acquiring the lock (and interruption of lock
                acquisition is supported)
        """
        ...


    def unlock(self) -> None:
        """
        Releases the lock.
        
        **Implementation Considerations**
        
        A `Lock` implementation will usually impose
        restrictions on which thread can release a lock (typically only the
        holder of the lock can release it) and may throw
        an (unchecked) exception if the restriction is violated.
        Any restrictions and the exception
        type must be documented by that `Lock` implementation.
        """
        ...


    def newCondition(self) -> "Condition":
        """
        Returns a new Condition instance that is bound to this
        `Lock` instance.
        
        Before waiting on the condition the lock must be held by the
        current thread.
        A call to Condition.await() will atomically release the lock
        before waiting and re-acquire the lock before the wait returns.
        
        **Implementation Considerations**
        
        The exact operation of the Condition instance depends on
        the `Lock` implementation and must be documented by that
        implementation.

        Returns
        - A new Condition instance for this `Lock` instance

        Raises
        - UnsupportedOperationException: if this `Lock`
                implementation does not support conditions
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.SingletonImmutableTable

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.collect import *
from typing import Any, Callable, Iterable, Tuple


class SingletonImmutableTable(ImmutableTable):
    """
    An implementation of ImmutableTable that holds a single cell.

    Author(s)
    - Gregory Kick
    """

    def column(self, columnKey: "C") -> "ImmutableMap"["R", "V"]:
        ...


    def columnMap(self) -> "ImmutableMap"["C", dict["R", "V"]]:
        ...


    def rowMap(self) -> "ImmutableMap"["R", dict["C", "V"]]:
        ...


    def size(self) -> int:
        ...



"""
Python module generated from Java source file org.bukkit.entity.ItemDisplay

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.entity import *
from org.bukkit.inventory import ItemStack
from typing import Any, Callable, Iterable, Tuple


class ItemDisplay(Display):
    """
    Represents an item display entity.
    """

    def getItemStack(self) -> "ItemStack":
        """
        Gets the displayed item stack.

        Returns
        - the displayed item stack
        """
        ...


    def setItemStack(self, item: "ItemStack") -> None:
        """
        Sets the displayed item stack.

        Arguments
        - item: the new item stack
        """
        ...


    def getItemDisplayTransform(self) -> "ItemDisplayTransform":
        """
        Gets the item display transform for this entity.
        
        Defaults to ItemDisplayTransform.FIXED.

        Returns
        - item display transform
        """
        ...


    def setItemDisplayTransform(self, display: "ItemDisplayTransform") -> None:
        """
        Sets the item display transform for this entity.
        
        Defaults to ItemDisplayTransform.FIXED.

        Arguments
        - display: new display
        """
        ...


    class ItemDisplayTransform(Enum):
        """
        Represents the item model transform to be applied to the displayed item.
        """

        NONE = 0
        THIRDPERSON_LEFTHAND = 1
        THIRDPERSON_RIGHTHAND = 2
        FIRSTPERSON_LEFTHAND = 3
        FIRSTPERSON_RIGHTHAND = 4
        HEAD = 5
        GUI = 6
        GROUND = 7
        FIXED = 8



"""
Python module generated from Java source file org.bukkit.inventory.meta.components.consumable.effects.ConsumablePlaySound

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Sound
from org.bukkit.inventory.meta.components.consumable.effects import *
from typing import Any, Callable, Iterable, Tuple


class ConsumablePlaySound(ConsumableEffect):
    """
    Represent a sound played when an item is consumed.
    """

    def getSound(self) -> "Sound":
        """
        Gets the sound to play on completion of the item's consumption.

        Returns
        - the sound
        """
        ...


    def setSound(self, sound: "Sound") -> None:
        """
        Sets the sound to play on completion of the item's consumption.

        Arguments
        - sound: sound
        """
        ...



"""
Python module generated from Java source file java.util.NoSuchElementException

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import *
from typing import Any, Callable, Iterable, Tuple


class NoSuchElementException(RuntimeException):
    """
    Thrown by various accessor methods to indicate that the element being requested
    does not exist.

    See
    - java.util.Iterator.next()

    Since
    - 1.0
    """

    def __init__(self):
        """
        Constructs a `NoSuchElementException` with `null`
        as its error message string.
        """
        ...


    def __init__(self, s: str, cause: "Throwable"):
        """
        Constructs a `NoSuchElementException` with the specified detail
        message and cause.

        Arguments
        - s: the detail message, or null
        - cause: the cause (which is saved for later retrieval by the
                     .getCause() method), or null

        Since
        - 15
        """
        ...


    def __init__(self, cause: "Throwable"):
        """
        Constructs a `NoSuchElementException` with the specified cause.
        The detail message is set to `(cause == null ? null :
        cause.toString())` (which typically contains the class and
        detail message of `cause`).

        Arguments
        - cause: the cause (which is saved for later retrieval by the
                     .getCause() method)

        Since
        - 15
        """
        ...


    def __init__(self, s: str):
        """
        Constructs a `NoSuchElementException`, saving a reference
        to the error message string `s` for later retrieval by the
        `getMessage` method.

        Arguments
        - s: the detail message.
        """
        ...



"""
Python module generated from Java source file com.google.common.graph.Network

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import Beta
from com.google.common.graph import *
from com.google.errorprone.annotations import DoNotMock
from java.util import Optional
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class Network(SuccessorsFunction, PredecessorsFunction):
    """
    An interface for <a
    href="https://en.wikipedia.org/wiki/Graph_(discrete_mathematics)">graph</a>-structured data,
    whose edges are unique objects.
    
    A graph is composed of a set of nodes and a set of edges connecting pairs of nodes.
    
    There are three primary interfaces provided to represent graphs. In order of increasing
    complexity they are: Graph, ValueGraph, and Network. You should generally
    prefer the simplest interface that satisfies your use case. See the <a
    href="https://github.com/google/guava/wiki/GraphsExplained#choosing-the-right-graph-type">
    "Choosing the right graph type"</a> section of the Guava User Guide for more details.
    
    <h3>Capabilities</h3>
    
    `Network` supports the following use cases (<a
    href="https://github.com/google/guava/wiki/GraphsExplained#definitions">definitions of
    terms</a>):
    
    
      - directed graphs
      - undirected graphs
      - graphs that do/don't allow parallel edges
      - graphs that do/don't allow self-loops
      - graphs whose nodes/edges are insertion-ordered, sorted, or unordered
      - graphs whose edges are unique objects
    
    
    <h3>Building a `Network`</h3>
    
    The implementation classes that `common.graph` provides are not public, by design. To
    create an instance of one of the built-in implementations of `Network`, use the NetworkBuilder class:
    
    ````MutableNetwork<Integer, MyEdge> network = NetworkBuilder.directed().build();````
    
    NetworkBuilder.build() returns an instance of MutableNetwork, which is a
    subtype of `Network` that provides methods for adding and removing nodes and edges. If you
    do not need to mutate a network (e.g. if you write a method than runs a read-only algorithm on
    the network), you should use the non-mutating Network interface, or an ImmutableNetwork.
    
    You can create an immutable copy of an existing `Network` using ImmutableNetwork.copyOf(Network):
    
    ````ImmutableNetwork<Integer, MyEdge> immutableGraph = ImmutableNetwork.copyOf(network);````
    
    Instances of ImmutableNetwork do not implement MutableNetwork (obviously!) and
    are contractually guaranteed to be unmodifiable and thread-safe.
    
    The Guava User Guide has <a
    href="https://github.com/google/guava/wiki/GraphsExplained#building-graph-instances">more
    information on (and examples of) building graphs</a>.
    
    <h3>Additional documentation</h3>
    
    See the Guava User Guide for the `common.graph` package (<a
    href="https://github.com/google/guava/wiki/GraphsExplained">"Graphs Explained"</a>) for
    additional documentation, including:
    
    
      - <a
          href="https://github.com/google/guava/wiki/GraphsExplained#equals-hashcode-and-graph-equivalence">
          `equals()`, `hashCode()`, and graph equivalence</a>
      - <a href="https://github.com/google/guava/wiki/GraphsExplained#synchronization">
          Synchronization policy</a>
      - <a href="https://github.com/google/guava/wiki/GraphsExplained#notes-for-implementors">Notes
          for implementors</a>
    
    
    Type `<N>`: Node parameter type
    
    Type `<E>`: Edge parameter type

    Author(s)
    - Joshua O'Madadhain

    Since
    - 20.0
    """

    def nodes(self) -> set["N"]:
        """
        Returns all nodes in this network, in the order specified by .nodeOrder().
        """
        ...


    def edges(self) -> set["E"]:
        """
        Returns all edges in this network, in the order specified by .edgeOrder().
        """
        ...


    def asGraph(self) -> "Graph"["N"]:
        """
        Returns a live view of this network as a Graph. The resulting Graph will have
        an edge connecting node A to node B if this Network has an edge connecting A to B.
        
        If this network .allowsParallelEdges() allows parallel edges, parallel edges will be
        treated as if collapsed into a single edge. For example, the .degree(Object) of a node
        in the Graph view may be less than the degree of the same node in this Network.
        """
        ...


    def isDirected(self) -> bool:
        """
        Returns True if the edges in this network are directed. Directed edges connect a EndpointPair.source() source node to a EndpointPair.target() target node, while
        undirected edges connect a pair of nodes to each other.
        """
        ...


    def allowsParallelEdges(self) -> bool:
        """
        Returns True if this network allows parallel edges. Attempting to add a parallel edge to a
        network that does not allow them will throw an IllegalArgumentException.
        """
        ...


    def allowsSelfLoops(self) -> bool:
        """
        Returns True if this network allows self-loops (edges that connect a node to itself).
        Attempting to add a self-loop to a network that does not allow them will throw an IllegalArgumentException.
        """
        ...


    def nodeOrder(self) -> "ElementOrder"["N"]:
        """
        Returns the order of iteration for the elements of .nodes().
        """
        ...


    def edgeOrder(self) -> "ElementOrder"["E"]:
        """
        Returns the order of iteration for the elements of .edges().
        """
        ...


    def adjacentNodes(self, node: "N") -> set["N"]:
        """
        Returns a live view of the nodes which have an incident edge in common with `node` in
        this graph.
        
        This is equal to the union of .predecessors(Object) and .successors(Object).
        
        If `node` is removed from the network after this method is called, the `Set`
        `view` returned by this method will be invalidated, and will throw `IllegalStateException` if it is accessed in any way, with the following exceptions:
        
        
          - `view.equals(view)` evaluates to `True` (but any other `equals()` expression
              involving `view` will throw)
          - `hashCode()` does not throw
          - if `node` is re-added to the network after having been removed, `view`'s
              behavior is undefined

        Raises
        - IllegalArgumentException: if `node` is not an element of this network
        """
        ...


    def predecessors(self, node: "N") -> set["N"]:
        """
        Returns a live view of all nodes in this network adjacent to `node` which can be reached
        by traversing `node`'s incoming edges *against* the direction (if any) of the edge.
        
        In an undirected network, this is equivalent to .adjacentNodes(Object).
        
        If `node` is removed from the network after this method is called, the `Set` returned
        by this method will be invalidated, and will throw `IllegalStateException` if it is accessed in
        any way.

        Raises
        - IllegalArgumentException: if `node` is not an element of this network
        """
        ...


    def successors(self, node: "N") -> set["N"]:
        """
        Returns a live view of all nodes in this network adjacent to `node` which can be reached
        by traversing `node`'s outgoing edges in the direction (if any) of the edge.
        
        In an undirected network, this is equivalent to .adjacentNodes(Object).
        
        This is *not* the same as "all nodes reachable from `node` by following outgoing
        edges". For that functionality, see Graphs.reachableNodes(Graph, Object).
        
        If `node` is removed from the network after this method is called, the `Set`
        `view` returned by this method will be invalidated, and will throw `IllegalStateException` if it is accessed in any way, with the following exceptions:
        
        
          - `view.equals(view)` evaluates to `True` (but any other `equals()` expression
              involving `view` will throw)
          - `hashCode()` does not throw
          - if `node` is re-added to the network after having been removed, `view`'s
              behavior is undefined

        Raises
        - IllegalArgumentException: if `node` is not an element of this network
        """
        ...


    def incidentEdges(self, node: "N") -> set["E"]:
        """
        Returns a live view of the edges whose .incidentNodes(Object) incident nodes in this
        network include `node`.
        
        This is equal to the union of .inEdges(Object) and .outEdges(Object).
        
        If `node` is removed from the network after this method is called, the `Set`
        `view` returned by this method will be invalidated, and will throw `IllegalStateException` if it is accessed in any way, with the following exceptions:
        
        
          - `view.equals(view)` evaluates to `True` (but any other `equals()` expression
              involving `view` will throw)
          - `hashCode()` does not throw
          - if `node` is re-added to the network after having been removed, `view`'s
              behavior is undefined

        Raises
        - IllegalArgumentException: if `node` is not an element of this network

        Since
        - 24.0
        """
        ...


    def inEdges(self, node: "N") -> set["E"]:
        """
        Returns a live view of all edges in this network which can be traversed in the direction (if
        any) of the edge to end at `node`.
        
        In a directed network, an incoming edge's EndpointPair.target() equals `node`.
        
        In an undirected network, this is equivalent to .incidentEdges(Object).
        
        If `node` is removed from the network after this method is called, the `Set`
        `view` returned by this method will be invalidated, and will throw `IllegalStateException` if it is accessed in any way, with the following exceptions:
        
        
          - `view.equals(view)` evaluates to `True` (but any other `equals()` expression
              involving `view` will throw)
          - `hashCode()` does not throw
          - if `node` is re-added to the network after having been removed, `view`'s
              behavior is undefined

        Raises
        - IllegalArgumentException: if `node` is not an element of this network
        """
        ...


    def outEdges(self, node: "N") -> set["E"]:
        """
        Returns a live view of all edges in this network which can be traversed in the direction (if
        any) of the edge starting from `node`.
        
        In a directed network, an outgoing edge's EndpointPair.source() equals `node`.
        
        In an undirected network, this is equivalent to .incidentEdges(Object).
        
        If `node` is removed from the network after this method is called, the `Set`
        `view` returned by this method will be invalidated, and will throw `IllegalStateException` if it is accessed in any way, with the following exceptions:
        
        
          - `view.equals(view)` evaluates to `True` (but any other `equals()` expression
              involving `view` will throw)
          - `hashCode()` does not throw
          - if `node` is re-added to the network after having been removed, `view`'s
              behavior is undefined

        Raises
        - IllegalArgumentException: if `node` is not an element of this network
        """
        ...


    def degree(self, node: "N") -> int:
        """
        Returns the count of `node`'s .incidentEdges(Object) incident edges, counting
        self-loops twice (equivalently, the number of times an edge touches `node`).
        
        For directed networks, this is equal to `inDegree(node) + outDegree(node)`.
        
        For undirected networks, this is equal to `incidentEdges(node).size()` + (number of
        self-loops incident to `node`).
        
        If the count is greater than `Integer.MAX_VALUE`, returns `Integer.MAX_VALUE`.

        Raises
        - IllegalArgumentException: if `node` is not an element of this network
        """
        ...


    def inDegree(self, node: "N") -> int:
        """
        Returns the count of `node`'s .inEdges(Object) incoming edges in a directed
        network. In an undirected network, returns the .degree(Object).
        
        If the count is greater than `Integer.MAX_VALUE`, returns `Integer.MAX_VALUE`.

        Raises
        - IllegalArgumentException: if `node` is not an element of this network
        """
        ...


    def outDegree(self, node: "N") -> int:
        """
        Returns the count of `node`'s .outEdges(Object) outgoing edges in a directed
        network. In an undirected network, returns the .degree(Object).
        
        If the count is greater than `Integer.MAX_VALUE`, returns `Integer.MAX_VALUE`.

        Raises
        - IllegalArgumentException: if `node` is not an element of this network
        """
        ...


    def incidentNodes(self, edge: "E") -> "EndpointPair"["N"]:
        """
        Returns the nodes which are the endpoints of `edge` in this network.

        Raises
        - IllegalArgumentException: if `edge` is not an element of this network
        """
        ...


    def adjacentEdges(self, edge: "E") -> set["E"]:
        """
        Returns a live view of the edges which have an .incidentNodes(Object) incident node in
        common with `edge`. An edge is not considered adjacent to itself.
        
        If `edge` is removed from the network after this method is called, the `Set`
        `view` returned by this method will be invalidated, and will throw `IllegalStateException` if it is accessed in any way, with the following exceptions:
        
        
          - `view.equals(view)` evaluates to `True` (but any other `equals()` expression
              involving `view` will throw)
          - `hashCode()` does not throw
          - if `edge` is re-added to the network after having been removed, `view`'s
              behavior is undefined

        Raises
        - IllegalArgumentException: if `edge` is not an element of this network
        """
        ...


    def edgesConnecting(self, nodeU: "N", nodeV: "N") -> set["E"]:
        """
        Returns a live view of the set of edges that each directly connect `nodeU` to `nodeV`.
        
        In an undirected network, this is equal to `edgesConnecting(nodeV, nodeU)`.
        
        The resulting set of edges will be parallel (i.e. have equal .incidentNodes(Object)). If this network does not .allowsParallelEdges() allow parallel
        edges, the resulting set will contain at most one edge (equivalent to `edgeConnecting(nodeU, nodeV).asSet()`).
        
        If either `nodeU` or `nodeV` are removed from the network after this method is
        called, the `Set` `view` returned by this method will be invalidated, and will
        throw `IllegalStateException` if it is accessed in any way, with the following
        exceptions:
        
        
          - `view.equals(view)` evaluates to `True` (but any other `equals()` expression
              involving `view` will throw)
          - `hashCode()` does not throw
          - if `nodeU` or `nodeV` are re-added to the network after having been removed,
              `view`'s behavior is undefined

        Raises
        - IllegalArgumentException: if `nodeU` or `nodeV` is not an element of this
            network
        """
        ...


    def edgesConnecting(self, endpoints: "EndpointPair"["N"]) -> set["E"]:
        """
        Returns a live view of the set of edges that each directly connect `endpoints` (in the
        order, if any, specified by `endpoints`).
        
        The resulting set of edges will be parallel (i.e. have equal .incidentNodes(Object)). If this network does not .allowsParallelEdges() allow parallel
        edges, the resulting set will contain at most one edge (equivalent to `edgeConnecting(endpoints).asSet()`).
        
        If this network is directed, `endpoints` must be ordered.
        
        If either element of `endpoints` is removed from the network after this method is
        called, the `Set` `view` returned by this method will be invalidated, and will
        throw `IllegalStateException` if it is accessed in any way, with the following
        exceptions:
        
        
          - `view.equals(view)` evaluates to `True` (but any other `equals()` expression
              involving `view` will throw)
          - `hashCode()` does not throw
          - if either endpoint is re-added to the network after having been removed, `view`'s
              behavior is undefined

        Raises
        - IllegalArgumentException: if either endpoint is not an element of this network
        - IllegalArgumentException: if the endpoints are unordered and the network is directed

        Since
        - 27.1
        """
        ...


    def edgeConnecting(self, nodeU: "N", nodeV: "N") -> "Optional"["E"]:
        """
        Returns the single edge that directly connects `nodeU` to `nodeV`, if one is
        present, or `Optional.empty()` if no such edge exists.
        
        In an undirected network, this is equal to `edgeConnecting(nodeV, nodeU)`.

        Raises
        - IllegalArgumentException: if there are multiple parallel edges connecting `nodeU`
            to `nodeV`
        - IllegalArgumentException: if `nodeU` or `nodeV` is not an element of this
            network

        Since
        - 23.0
        """
        ...


    def edgeConnecting(self, endpoints: "EndpointPair"["N"]) -> "Optional"["E"]:
        """
        Returns the single edge that directly connects `endpoints` (in the order, if any,
        specified by `endpoints`), if one is present, or `Optional.empty()` if no such edge
        exists.
        
        If this network is directed, the endpoints must be ordered.

        Raises
        - IllegalArgumentException: if there are multiple parallel edges connecting `nodeU`
            to `nodeV`
        - IllegalArgumentException: if either endpoint is not an element of this network
        - IllegalArgumentException: if the endpoints are unordered and the network is directed

        Since
        - 27.1
        """
        ...


    def edgeConnectingOrNull(self, nodeU: "N", nodeV: "N") -> "E":
        """
        Returns the single edge that directly connects `nodeU` to `nodeV`, if one is
        present, or `null` if no such edge exists.
        
        In an undirected network, this is equal to `edgeConnectingOrNull(nodeV, nodeU)`.

        Raises
        - IllegalArgumentException: if there are multiple parallel edges connecting `nodeU`
            to `nodeV`
        - IllegalArgumentException: if `nodeU` or `nodeV` is not an element of this
            network

        Since
        - 23.0
        """
        ...


    def edgeConnectingOrNull(self, endpoints: "EndpointPair"["N"]) -> "E":
        """
        Returns the single edge that directly connects `endpoints` (in the order, if any,
        specified by `endpoints`), if one is present, or `null` if no such edge exists.
        
        If this network is directed, the endpoints must be ordered.

        Raises
        - IllegalArgumentException: if there are multiple parallel edges connecting `nodeU`
            to `nodeV`
        - IllegalArgumentException: if either endpoint is not an element of this network
        - IllegalArgumentException: if the endpoints are unordered and the network is directed

        Since
        - 27.1
        """
        ...


    def hasEdgeConnecting(self, nodeU: "N", nodeV: "N") -> bool:
        """
        Returns True if there is an edge that directly connects `nodeU` to `nodeV`. This is
        equivalent to `nodes().contains(nodeU) && successors(nodeU).contains(nodeV)`, and to
        `edgeConnectingOrNull(nodeU, nodeV) != null`.
        
        In an undirected network, this is equal to `hasEdgeConnecting(nodeV, nodeU)`.

        Since
        - 23.0
        """
        ...


    def hasEdgeConnecting(self, endpoints: "EndpointPair"["N"]) -> bool:
        """
        Returns True if there is an edge that directly connects `endpoints` (in the order, if
        any, specified by `endpoints`).
        
        Unlike the other `EndpointPair`-accepting methods, this method does not throw if the
        endpoints are unordered and the network is directed; it simply returns `False`. This is
        for consistency with Graph.hasEdgeConnecting(EndpointPair) and ValueGraph.hasEdgeConnecting(EndpointPair).

        Since
        - 27.1
        """
        ...


    def equals(self, object: "Object") -> bool:
        """
        Returns `True` iff `object` is a Network that has the same elements and the
        same structural relationships as those in this network.
        
        Thus, two networks A and B are equal if **all** of the following are True:
        
        
          - A and B have equal .isDirected() directedness.
          - A and B have equal .nodes() node sets.
          - A and B have equal .edges() edge sets.
          - Every edge in A and B connects the same nodes in the same direction (if any).
        
        
        Network properties besides .isDirected() directedness do **not** affect equality.
        For example, two networks may be considered equal even if one allows parallel edges and the
        other doesn't. Additionally, the order in which nodes or edges are added to the network, and
        the order in which they are iterated over, are irrelevant.
        
        A reference implementation of this is provided by AbstractNetwork.equals(Object).
        """
        ...


    def hashCode(self) -> int:
        """
        Returns the hash code for this network. The hash code of a network is defined as the hash code
        of a map from each of its .edges() edges to their .incidentNodes(Object)
        incident nodes.
        
        A reference implementation of this is provided by AbstractNetwork.hashCode().
        """
        ...



"""
Python module generated from Java source file com.google.common.hash.Striped64

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.hash import *
from java.lang.reflect import Field
from java.security import AccessController
from java.security import PrivilegedActionException
from java.security import PrivilegedExceptionAction
from java.util import Random
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from sun.misc import Unsafe
from typing import Any, Callable, Iterable, Tuple


class Striped64(Number):
    """
    A package-local class holding common representation and mechanics for classes supporting dynamic
    striping on 64bit values. The class extends Number so that concrete subclasses must publicly do
    so.
    """





"""
Python module generated from Java source file org.bukkit.block.Conduit

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import *
from org.bukkit.entity import LivingEntity
from org.bukkit.util import BoundingBox
from typing import Any, Callable, Iterable, Tuple


class Conduit(TileState):
    """
    Represents a captured state of a conduit.
    """

    def isActive(self) -> bool:
        """
        Checks whether or not this conduit is active.
        
        A conduit is considered active if there are at least 16 valid frame
        blocks surrounding it and the conduit is surrounded by a 3x3x3 area of
        water source blocks (or waterlogged blocks), at which point its animation
        will activate, start spinning, and apply effects to nearby players.

        Returns
        - True if active, False otherwise
        """
        ...


    def isHunting(self) -> bool:
        """
        Get whether or not this conduit is actively hunting for nearby hostile
        creatures.
        
        A conduit will hunt if it is active (see .isActive()) and its
        frame is complete (it is surrounded by at least 42 valid frame blocks).
        While hunting, the .getTarget()
        conduit's target, if within its .getHuntingArea() hunting area,
        will be damaged every 2 seconds.

        Returns
        - True if hunting, False otherwise
        """
        ...


    def getFrameBlocks(self) -> Iterable["Block"]:
        """
        Get a Collection of all Block Blocks that make up the
        frame of this conduit. The returned collection will contain only blocks
        that match the types required by the conduit to make up a valid frame,
        <strong>not</strong> the blocks at which the conduit is searching,
        meaning it will be of variable size depending on how many valid frames
        are surrounding the conduit at the time of invocation.

        Returns
        - the frame blocks
        """
        ...


    def getFrameBlockCount(self) -> int:
        """
        Get the amount of valid frame blocks that are currently surrounding the
        conduit.

        Returns
        - the frame block count
        """
        ...


    def getRange(self) -> int:
        """
        Get the range (measured in blocks) within which players will receive the
        conduit's benefits.

        Returns
        - the conduit range
        """
        ...


    def setTarget(self, target: "LivingEntity") -> bool:
        """
        Set the conduit's hunting target.
        
        Note that the value set by this method may be overwritten by the
        conduit's periodic hunting logic. If the target is ever set to
        `null`, the conduit will continue to look for a new target.
        Additionally, if the target is set to an entity that does not meet a
        conduit's hunting conditions (e.g. the entity is not within the
        .getHuntingArea() hunting area, has already been killed, etc.)
        then the passed entity will be ignored and the conduit will also continue
        to look for a new target.

        Arguments
        - target: the target entity, or null to remove the target

        Returns
        - True if the target was changed, False if the target was the same
        """
        ...


    def getTarget(self) -> "LivingEntity":
        """
        Get the conduit's hunting target.

        Returns
        - the hunting target, or null if the conduit does not have a target
        """
        ...


    def hasTarget(self) -> bool:
        """
        Check whether or not this conduit has an active (alive) hunting target.

        Returns
        - True if has a hunting target, False otherwise
        """
        ...


    def getHuntingArea(self) -> "BoundingBox":
        """
        Get a BoundingBox (relative to real-world coordinates) in which
        the conduit will search for hostile entities to target.

        Returns
        - the hunting area bounding box
        """
        ...



"""
Python module generated from Java source file org.joml.sampling.Convolution

Java source file obtained from artifact joml version 1.10.8

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.joml import Math
from org.joml.sampling import *
from typing import Any, Callable, Iterable, Tuple


class Convolution:
    """
    Generates various convolution kernels.

    Author(s)
    - Kai Burjack
    """

    @staticmethod
    def gaussianKernel(rows: int, cols: int, sigma: float, dest: "FloatBuffer") -> None:
        """
        Generate a Gaussian convolution kernel with the given number of rows and columns, and store
        the factors in row-major order in `dest`.

        Arguments
        - rows: the number of rows (must be an odd number)
        - cols: the number of columns (must be an odd number)
        - sigma: the standard deviation of the filter kernel values
        - dest: will hold the kernel factors in row-major order
        """
        ...


    @staticmethod
    def gaussianKernel(rows: int, cols: int, sigma: float, dest: list[float]) -> None:
        """
        Generate a Gaussian convolution kernel with the given number of rows and columns, and store
        the factors in row-major order in `dest`.

        Arguments
        - rows: the number of rows (must be an odd number)
        - cols: the number of columns (must be an odd number)
        - sigma: the standard deviation of the filter kernel values
        - dest: will hold the kernel factors in row-major order
        """
        ...



"""
Python module generated from Java source file com.google.common.util.concurrent.AbstractExecutionThreadService

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.util.concurrent import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.time import Duration
from java.util.concurrent import Executor
from java.util.concurrent import TimeUnit
from java.util.concurrent import TimeoutException
from typing import Any, Callable, Iterable, Tuple


class AbstractExecutionThreadService(Service):
    """
    Base class for services that can implement .startUp, .run and .shutDown
    methods. This class uses a single thread to execute the service; consider AbstractService
    if you would like to manage any threading manually.

    Author(s)
    - Jesse Wilson

    Since
    - 1.0
    """

    def toString(self) -> str:
        ...


    def isRunning(self) -> bool:
        ...


    def state(self) -> "State":
        ...


    def addListener(self, listener: "Listener", executor: "Executor") -> None:
        """
        Since
        - 13.0
        """
        ...


    def failureCause(self) -> "Throwable":
        """
        Since
        - 14.0
        """
        ...


    def startAsync(self) -> "Service":
        """
        Since
        - 15.0
        """
        ...


    def stopAsync(self) -> "Service":
        """
        Since
        - 15.0
        """
        ...


    def awaitRunning(self) -> None:
        """
        Since
        - 15.0
        """
        ...


    def awaitRunning(self, timeout: "Duration") -> None:
        """
        Since
        - 28.0
        """
        ...


    def awaitRunning(self, timeout: int, unit: "TimeUnit") -> None:
        """
        Since
        - 15.0
        """
        ...


    def awaitTerminated(self) -> None:
        """
        Since
        - 15.0
        """
        ...


    def awaitTerminated(self, timeout: "Duration") -> None:
        """
        Since
        - 28.0
        """
        ...


    def awaitTerminated(self, timeout: int, unit: "TimeUnit") -> None:
        """
        Since
        - 15.0
        """
        ...



"""
Python module generated from Java source file java.util.concurrent.Executor

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util.concurrent import *
from typing import Any, Callable, Iterable, Tuple


class Executor:
    """
    An object that executes submitted Runnable tasks. This
    interface provides a way of decoupling task submission from the
    mechanics of how each task will be run, including details of thread
    use, scheduling, etc.  An `Executor` is normally used
    instead of explicitly creating threads. For example, rather than
    invoking `new Thread(new RunnableTask()).start()` for each
    of a set of tasks, you might use:
    
    ``` `Executor executor = anExecutor();
    executor.execute(new RunnableTask1());
    executor.execute(new RunnableTask2());
    ...````
    
    However, the `Executor` interface does not strictly require
    that execution be asynchronous. In the simplest case, an executor
    can run the submitted task immediately in the caller's thread:
    
    ``` `class DirectExecutor implements Executor {
      public void execute(Runnable r) {
        r.run();`
    }}```
    
    More typically, tasks are executed in some thread other than the
    caller's thread.  The executor below spawns a new thread for each
    task.
    
    ``` `class ThreadPerTaskExecutor implements Executor {
      public void execute(Runnable r) {
        new Thread(r).start();`
    }}```
    
    Many `Executor` implementations impose some sort of
    limitation on how and when tasks are scheduled.  The executor below
    serializes the submission of tasks to a second executor,
    illustrating a composite executor.
    
    ``` `class SerialExecutor implements Executor {
      final Queue<Runnable> tasks = new ArrayDeque<>();
      final Executor executor;
      Runnable active;
    
      SerialExecutor(Executor executor) {
        this.executor = executor;`
    
      public synchronized void execute(Runnable r) {
        tasks.add(() -> {
          try {
            r.run();
          } finally {
            scheduleNext();
          }
        });
        if (active == null) {
          scheduleNext();
        }
      }
    
      protected synchronized void scheduleNext() {
        if ((active = tasks.poll()) != null) {
          executor.execute(active);
        }
      }
    }}```
    
    The `Executor` implementations provided in this package
    implement ExecutorService, which is a more extensive
    interface.  The ThreadPoolExecutor class provides an
    extensible thread pool implementation. The Executors class
    provides convenient factory methods for these Executors.
    
    Memory consistency effects: Actions in a thread prior to
    submitting a `Runnable` object to an `Executor`
    <a href="package-summary.html#MemoryVisibility">*happen-before*</a>
    its execution begins, perhaps in another thread.

    Author(s)
    - Doug Lea

    Since
    - 1.5
    """

    def execute(self, command: "Runnable") -> None:
        """
        Executes the given command at some time in the future.  The command
        may execute in a new thread, in a pooled thread, or in the calling
        thread, at the discretion of the `Executor` implementation.

        Arguments
        - command: the runnable task

        Raises
        - RejectedExecutionException: if this task cannot be
        accepted for execution
        - NullPointerException: if command is null
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.ConcurrentHashMultiset

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.annotations import VisibleForTesting
from com.google.common.collect import *
from com.google.common.collect.Serialization import FieldSetter
from com.google.common.math import IntMath
from com.google.common.primitives import Ints
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.j2objc.annotations import WeakOuter
from java.io import IOException
from java.io import ObjectInputStream
from java.io import ObjectOutputStream
from java.io import Serializable
from java.util import Iterator
from java.util.concurrent import ConcurrentHashMap
from java.util.concurrent import ConcurrentMap
from java.util.concurrent.atomic import AtomicInteger
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class ConcurrentHashMultiset(AbstractMultiset, Serializable):
    """
    A multiset that supports concurrent modifications and that provides atomic versions of most
    `Multiset` operations (exceptions where noted). Null elements are not supported.
    
    See the Guava User Guide article on <a href=
    "https://github.com/google/guava/wiki/NewCollectionTypesExplained#multiset">`Multiset`</a>.

    Author(s)
    - mike nonemacher

    Since
    - 2.0
    """

    @staticmethod
    def create() -> "ConcurrentHashMultiset"["E"]:
        """
        Creates a new, empty `ConcurrentHashMultiset` using the default initial capacity, load
        factor, and concurrency settings.
        """
        ...


    @staticmethod
    def create(elements: Iterable["E"]) -> "ConcurrentHashMultiset"["E"]:
        """
        Creates a new `ConcurrentHashMultiset` containing the specified elements, using the
        default initial capacity, load factor, and concurrency settings.
        
        This implementation is highly efficient when `elements` is itself a Multiset.

        Arguments
        - elements: the elements that the multiset should contain
        """
        ...


    @staticmethod
    def create(countMap: "ConcurrentMap"["E", "AtomicInteger"]) -> "ConcurrentHashMultiset"["E"]:
        """
        Creates a new, empty `ConcurrentHashMultiset` using `countMap` as the internal
        backing map.
        
        This instance will assume ownership of `countMap`, and other code should not maintain
        references to the map or modify it in any way.
        
        The returned multiset is serializable if the input map is.

        Arguments
        - countMap: backing map for storing the elements in the multiset and their counts. It must
            be empty.

        Raises
        - IllegalArgumentException: if `countMap` is not empty

        Since
        - 20.0
        """
        ...


    def count(self, element: "Object") -> int:
        """
        Returns the number of occurrences of `element` in this multiset.

        Arguments
        - element: the element to look for

        Returns
        - the nonnegative number of occurrences of the element
        """
        ...


    def size(self) -> int:
        """
        
        
        If the data in the multiset is modified by any other threads during this method, it is
        undefined which (if any) of these modifications will be reflected in the result.
        """
        ...


    def toArray(self) -> list["Object"]:
        ...


    def toArray(self, array: list["T"]) -> list["T"]:
        ...


    def add(self, element: "E", occurrences: int) -> int:
        """
        Adds a number of occurrences of the specified element to this multiset.

        Arguments
        - element: the element to add
        - occurrences: the number of occurrences to add

        Returns
        - the previous count of the element before the operation; possibly zero

        Raises
        - IllegalArgumentException: if `occurrences` is negative, or if the resulting amount
            would exceed Integer.MAX_VALUE
        """
        ...


    def remove(self, element: "Object", occurrences: int) -> int:
        ...


    def removeExactly(self, element: "Object", occurrences: int) -> bool:
        """
        Removes exactly the specified number of occurrences of `element`, or makes no change if
        this is not possible.
        
        This method, in contrast to .remove(Object, int), has no effect when the element
        count is smaller than `occurrences`.

        Arguments
        - element: the element to remove
        - occurrences: the number of occurrences of `element` to remove

        Returns
        - `True` if the removal was possible (including if `occurrences` is zero)

        Raises
        - IllegalArgumentException: if `occurrences` is negative
        """
        ...


    def setCount(self, element: "E", count: int) -> int:
        """
        Adds or removes occurrences of `element` such that the .count of the element
        becomes `count`.

        Returns
        - the count of `element` in the multiset before this call

        Raises
        - IllegalArgumentException: if `count` is negative
        """
        ...


    def setCount(self, element: "E", expectedOldCount: int, newCount: int) -> bool:
        """
        Sets the number of occurrences of `element` to `newCount`, but only if the count is
        currently `expectedOldCount`. If `element` does not appear in the multiset exactly
        `expectedOldCount` times, no changes will be made.

        Returns
        - `True` if the change was successful. This usually indicates that the multiset has
            been modified, but not always: in the case that `expectedOldCount == newCount`, the
            method will return `True` if the condition was met.

        Raises
        - IllegalArgumentException: if `expectedOldCount` or `newCount` is negative
        """
        ...


    def createEntrySet(self) -> set["Multiset.Entry"["E"]]:
        """
        Deprecated
        - Internal method, use .entrySet().
        """
        ...


    def isEmpty(self) -> bool:
        ...


    def iterator(self) -> Iterator["E"]:
        ...


    def clear(self) -> None:
        ...



"""
Python module generated from Java source file org.bukkit.block.structure.UsageMode

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.block.structure import *
from typing import Any, Callable, Iterable, Tuple


class UsageMode(Enum):
    """
    Represents how a org.bukkit.block.Structure can be used.
    """

    SAVE = 0
    """
    The mode used when saving a structure.
    """
    LOAD = 1
    """
    The mode used when loading a structure.
    """
    CORNER = 2
    """
    Used when saving a structure for easy size calculation. When using this
    mode, the Structure name MUST match the name in the second Structure
    block that is in UsageMode.SAVE.
    """
    DATA = 3
    """
    Used to run specific custom functions, which can only be used for certain
    Structures. The structure block is removed after this function completes.
    The data tags (functions) can be found on the
    <a href="https://minecraft.wiki/w/Structure_Block#Data">wiki</a>.
    """



"""
Python module generated from Java source file org.bukkit.persistence.PersistentDataContainer

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import NamespacedKey
from org.bukkit.persistence import *
from typing import Any, Callable, Iterable, Tuple


class PersistentDataContainer:
    """
    This interface represents a map like object, capable of storing custom tags
    in it.
    """

    def set(self, key: "NamespacedKey", type: "PersistentDataType"["P", "C"], value: "C") -> None:
        """
        Stores a metadata value on the PersistentDataHolder instance.
        
        This API cannot be used to manipulate minecraft data, as the values will
        be stored using your namespace. This method will override any existing
        value the PersistentDataHolder may have stored under the provided
        key.
        
        Type `<P>`: the generic java type of the tag value
        
        Type `<C>`: the generic type of the object to store

        Arguments
        - key: the key this value will be stored under
        - type: the type this tag uses
        - value: the value to store in the tag

        Raises
        - IllegalArgumentException: if the key is null
        - IllegalArgumentException: if the type is null
        - IllegalArgumentException: if the value is null. Removing a tag should
        be done using .remove(NamespacedKey)
        - IllegalArgumentException: if no suitable adapter was found for
        the PersistentDataType.getPrimitiveType()
        """
        ...


    def has(self, key: "NamespacedKey", type: "PersistentDataType"["P", "C"]) -> bool:
        """
        Returns if the persistent metadata provider has metadata registered
        matching the provided parameters.
        
        This method will only return True if the found value has the same primitive
        data type as the provided key.
        
        Storing a value using a custom PersistentDataType implementation
        will not store the complex data type. Therefore storing a UUID (by
        storing a byte[]) will match has("key" ,
        PersistentDataType.BYTE_ARRAY). Likewise a stored byte[] will
        always match your UUID PersistentDataType even if it is not 16
        bytes long.
        
        This method is only usable for custom object keys. Overwriting existing
        tags, like the display name, will not work as the values are stored
        using your namespace.
        
        Type `<P>`: the generic type of the stored primitive
        
        Type `<C>`: the generic type of the eventually created complex object

        Arguments
        - key: the key the value is stored under
        - type: the type the primative stored value has to match

        Returns
        - if a value with the provided key and type exists

        Raises
        - IllegalArgumentException: if the key to look up is null
        - IllegalArgumentException: if the type to cast the found object to is
        null
        """
        ...


    def has(self, key: "NamespacedKey") -> bool:
        """
        Returns if the persistent metadata provider has metadata registered matching
        the provided parameters.
        
        This method will return True as long as a value with the given key exists,
        regardless of its type.
        
        This method is only usable for custom object keys. Overwriting existing tags,
        like the display name, will not work as the values are stored using your
        namespace.

        Arguments
        - key: the key the value is stored under

        Returns
        - if a value with the provided key exists

        Raises
        - IllegalArgumentException: if the key to look up is null
        """
        ...


    def get(self, key: "NamespacedKey", type: "PersistentDataType"["P", "C"]) -> "C":
        """
        Returns the metadata value that is stored on the
        PersistentDataHolder instance.
        
        Type `<P>`: the generic type of the stored primitive
        
        Type `<C>`: the generic type of the eventually created complex object

        Arguments
        - key: the key to look up in the custom tag map
        - type: the type the value must have and will be casted to

        Returns
        - the value or `null` if no value was mapped under the given
        value

        Raises
        - IllegalArgumentException: if the key to look up is null
        - IllegalArgumentException: if the type to cast the found object to is
        null
        - IllegalArgumentException: if a value exists under the given key,
        but cannot be accessed using the given type
        - IllegalArgumentException: if no suitable adapter was found for
        the PersistentDataType.getPrimitiveType()
        """
        ...


    def getOrDefault(self, key: "NamespacedKey", type: "PersistentDataType"["P", "C"], defaultValue: "C") -> "C":
        """
        Returns the metadata value that is stored on the
        PersistentDataHolder instance. If the value does not exist in the
        container, the default value provided is returned.
        
        Type `<P>`: the generic type of the stored primitive
        
        Type `<C>`: the generic type of the eventually created complex object

        Arguments
        - key: the key to look up in the custom tag map
        - type: the type the value must have and will be casted to
        - defaultValue: the default value to return if no value was found for
        the provided key

        Returns
        - the value or the default value if no value was mapped under the
        given key

        Raises
        - IllegalArgumentException: if the key to look up is null
        - IllegalArgumentException: if the type to cast the found object to is
        null
        - IllegalArgumentException: if a value exists under the given key,
        but cannot be accessed using the given type
        - IllegalArgumentException: if no suitable adapter was found for
        the PersistentDataType.getPrimitiveType()
        """
        ...


    def getKeys(self) -> set["NamespacedKey"]:
        """
        Get the set of keys present on this PersistentDataContainer
        instance.
        
        Any changes made to the returned set will not be reflected on the
        instance.

        Returns
        - the key set
        """
        ...


    def remove(self, key: "NamespacedKey") -> None:
        """
        Removes a custom key from the PersistentDataHolder instance.

        Arguments
        - key: the key to remove

        Raises
        - IllegalArgumentException: if the provided key is null
        """
        ...


    def isEmpty(self) -> bool:
        """
        Returns if the container instance is empty, therefore has no entries
        inside it.

        Returns
        - the boolean
        """
        ...


    def copyTo(self, other: "PersistentDataContainer", replace: bool) -> None:
        """
        Copies all values from this PersistentDataContainer to the provided
        container.
        
        This method only copies custom object keys. Existing tags, like the display
        name, will not be copied as the values are stored using your namespace.

        Arguments
        - other: the container to copy to
        - replace: whether to replace any matching values in the target container

        Raises
        - IllegalArgumentException: if the other container is null
        """
        ...


    def getAdapterContext(self) -> "PersistentDataAdapterContext":
        """
        Returns the adapter context this tag container uses.

        Returns
        - the tag context
        """
        ...
